import pytest

from insights.parsers.initscript import InitScript, EmptyFileException, NotInitscriptException
from insights.tests import context_wrap


NOTINITSCRIPT_SCRIPT = "etc/rc.d/init.d/script_notinitscript"
NOTINITSCRIPT_CONTENT = """
#! /bin/bash

echo script_notinitscript
""".strip()

CHKCONFIG_SCRIPT = "etc/rc.d/init.d/script_chkconfig"
CHKCONFIG_CONTENT = """
#!/bin/sh
#
# rhnsd:        Starts the Spacewalk Daemon
#
# chkconfig: 345 97 03
# description:  This is a daemon which handles the task of connecting \
#               periodically to the Spacewalk servers to \
#               check for updates, notifications and perform system \
#               monitoring tasks according to the service level that \
#               this server is subscribed for
#
# processname: rhnsd
# pidfile: /var/run/rhnsd.pid
#

echo script_chkconfig
""".strip()

LSB_SCRIPT = "etc/rc.d/init.d/script_lsb"
LSB_CONTENT = """
#!/bin/sh
### BEGIN INIT INFO
# Provides: rhnsd
# Required-Start: $local_fs $network $remote_fs $named $time
# Required-Stop: $local_fs $network $remote_fs $named
# Default-Start: 2 3 4 5
# Default-Stop: 0 1 6
# Short-Description: Starts the Spacewalk Daemon
# Description: This is a daemon which handles the task of connecting
#               periodically to the Spacewalk servers to
#               check for updates, notifications and perform system
#               monitoring tasks according to the service level that
#               this server is subscribed for.
### END INIT INFO

echo script_lsb
""".strip()

CHKCONFIG_LSB_SCRIPT = "etc/rc.d/init.d/script_chkconfig+lsb"
CHKCONFIG_LSB_CONTENT = """
#!/bin/sh
#
# rhnsd:        Starts the Spacewalk Daemon
#
# chkconfig: 345 97 03
# description:  This is a daemon which handles the task of connecting \
#               periodically to the Spacewalk servers to \
#               check for updates, notifications and perform system \
#               monitoring tasks according to the service level that \
#               this server is subscribed for
#
# processname: rhnsd
# pidfile: /var/run/rhnsd.pid
#

### BEGIN INIT INFO
# Provides: rhnsd
# Required-Start: $local_fs $network $remote_fs $named $time
# Required-Stop: $local_fs $network $remote_fs $named
# Default-Start: 2 3 4 5
# Default-Stop: 0 1 6
# Short-Description: Starts the Spacewalk Daemon
# Description: This is a daemon which handles the task of connecting
#               periodically to the Spacewalk servers to
#               check for updates, notifications and perform system
#               monitoring tasks according to the service level that
#               this server is subscribed for.
### END INIT INFO

echo script_chkconfig+lsb
""".strip()

EMPTY_SCRIPT = "etc/rc.d/init.d/script_empty"
EMPTY_CONTENT = """
""".strip()

HINTSONLY_SCRIPT = "etc/rc.d/init.d/script_hintsonly"
HINTSONLY_CONTENT = """
#!/bin/sh
#

case "$1" in
start)
        ;;
stop)
        ;;
esac
""".strip()


COMMENTS_SCRIPT = "etc/rc.d/init.d/script_comments"
COMMENTS_CONTENT = """
#! bin/broken
#
#
#case "$1" in
#start)
#        ;;
#stop)
#        ;;
#esac
""".strip()


def test_initscript1():
    context = context_wrap(NOTINITSCRIPT_CONTENT, path=NOTINITSCRIPT_SCRIPT)
    with pytest.raises(NotInitscriptException) as e_info:
        InitScript(context)
    assert context.path in str(e_info.value)
    assert "confidence: 1" in str(e_info.value)


def test_initscript2():
    context = context_wrap(CHKCONFIG_CONTENT, path=CHKCONFIG_SCRIPT)
    r = InitScript(context)
    assert r.file_name == "script_chkconfig"


def test_initscript3():
    context = context_wrap(LSB_CONTENT, path=LSB_SCRIPT)
    r = InitScript(context)
    assert r.file_name == "script_lsb"


def test_initscript4():
    context = context_wrap(CHKCONFIG_LSB_CONTENT, path=CHKCONFIG_LSB_SCRIPT)
    r = InitScript(context)
    assert r.file_name == "script_chkconfig+lsb"


def test_initscript5():
    context = context_wrap(HINTSONLY_CONTENT, path=HINTSONLY_SCRIPT)
    r = InitScript(context)
    assert r.file_name == "script_hintsonly"


def test_initscript6():
    context = context_wrap(EMPTY_CONTENT, path=EMPTY_SCRIPT)
    with pytest.raises(EmptyFileException) as e_info:
        InitScript(context)
    assert context.path in str(e_info.value)


def test_initscript7():
    context = context_wrap(COMMENTS_CONTENT, path=COMMENTS_SCRIPT)
    with pytest.raises(NotInitscriptException) as e_info:
        InitScript(context)
    assert context.path in str(e_info.value)
    assert "confidence: 0" in str(e_info.value)



import doctest
from datetime import datetime

from insights.parsers import mistral_log
from insights.tests import context_wrap

EXECUTOR_LOG_CONTENT = """
2019-03-01 13:54:41.091 26749 DEBUG oslo_concurrency.lockutils [-] Acquired semaphore "singleton_lock" lock /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:212
2019-03-01 13:54:41.091 26749 DEBUG oslo_concurrency.lockutils [-] Releasing semaphore "singleton_lock" lock /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:228
2019-03-01 13:54:41.092 26749 DEBUG oslo_service.service [-] Full set of CONF: _wait_for_exit_or_signal /usr/lib/python2.7/site-packages/oslo_service/service.py:303
2019-03-01 13:54:41.092 26749 DEBUG oslo_service.service [-] ******************************************************************************** log_opt_values /usr/lib/python2.7/site-packages/oslo_config/cfg.py:28
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor Traceback (most recent call last):
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor   File "/usr/lib/python2.7/site-packages/mistral/executors/default_executor.py", line 114, in run_action
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor     result = action.run(action_ctx)
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor   File "/usr/lib/python2.7/site-packages/mistral/actions/openstack/base.py", line 130, in run
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor     (self.__class__.__name__, self.client_method_name, str(e))
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor ActionException: HeatAction.stacks.get failed: ERROR: The Stack (overcloud) could not be found.
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor
2019-03-01 14:36:55.227 26749 INFO mistral.executors.executor_server [req-acd6d78c-9f9e-4d6c-9d6c-09717635ac29 89665ff87df144128ada2b8c59ec62f1 e4d0868a0715411fa1481f8ccfb61414 - default default] Received RPC request 'run_action'[action_ex_id=68693a02-8817-409e-a8d9-3572e7773da3, action_cls_str=mistral.actions.openstack.actions.IronicAction, action_cls_attrs={u'client_method_name': u'node.list'}, params={associated: True}, timeout=None]
""".strip()

SAMPLE_LOG_CONTENT = """
2019-03-01 13:54:41.091 26749 DEBUG oslo_concurrency.lockutils [-] Acquired semaphore "singleton_lock" lock /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:212
2019-03-01 13:54:41.091 26749 DEBUG oslo_concurrency.lockutils [-] Releasing semaphore "singleton_lock" lock /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:228
2019-03-01 13:54:41.092 26749 DEBUG oslo_service.service [-] Full set of CONF: _wait_for_exit_or_signal /usr/lib/python2.7/site-packages/oslo_service/service.py:303
2019-03-01 13:54:41.133 26749 DEBUG oslo_concurrency.lockutils [-] Lock "service_coordinator" released by "mistral.service.coordination.register_membership" :: held 0.000s inner /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:285
2019-03-01 13:56:05.329 26749 INFO mistral.executors.executor_server [req-a1d40531-b3b9-4fdf-802f-0371bf364551 89665ff87df144128ada2b8c59ec62f1 e4d0868a0715411fa1481f8ccfb61414 - default default] Received RPC request 'run_action'[action_ex_id=7c2c79ba-d7c4-4c4d-9e51-684bab00e6a9, action_cls_str=mistral.actions.std_actions.NoOpAction, action_cls_attrs={}, params={}, timeout=None]
2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor ActionException: HeatAction.stacks.get failed: ERROR: The Stack (overcloud) could not be found.
""".strip()


def test_mistral_executor_log():
    log = mistral_log.MistralExecutorLog(context_wrap(EXECUTOR_LOG_CONTENT))
    assert len(log.get('ERROR')) == 7
    assert log.get('oslo_concurrency.lockutils')[0]['raw_message'] == EXECUTOR_LOG_CONTENT.splitlines()[0]
    assert len(list(log.get_after(datetime(2019, 3, 1, 14, 30, 0)))) == 8


def test_documentation():
    failed_count, tests = doctest.testmod(
        mistral_log,
        globs={'executor_log': mistral_log.MistralExecutorLog(context_wrap(SAMPLE_LOG_CONTENT))}
    )
    assert failed_count == 0



import json
import os
from collections import defaultdict

import pytest
from mock.mock import patch

from insights import collect
from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core import dr
from insights.core.context import HostContext
from insights.core.plugins import datasource
from insights.core.spec_factory import command_with_args
from insights.core.spec_factory import RegistryPoint
from insights.core.spec_factory import SpecSet
from insights.parsers.hostname import Hostname
from insights.util.hostname import determine_hostname

orig_hostname = determine_hostname().split('.')[0]
base_path = "/tmp/empty_insights_unittest_"
# file path include the original hostname which is not obfuscated
tmp_file_path = "{0}{1}".format(base_path, orig_hostname)
collected_file = "localhost_empty"

specs_manifest = """
---
version: 0

client:
    context:
        class: insights.core.context.HostContext
        args:
            timeout: 5

    # commands and files to ignore
    blacklist:
        files: []
        commands: []
        patterns: []
        keywords: []

    persist:
        # add Specs.hostname here, as only 'persist' items will be collected
        - name: insights.specs.Specs.hostname
          enabled: true

        - name: insights.tests.test_specs_runtime_ds_obfuscation.Specs
          enabled: true

    run_strategy:
        name: serial
        args:
            max_workers: null

plugins:
    default_component_enabled: false

    packages:
        - insights.tests.test_specs_runtime_ds_obfuscation

    configs:
        - name: insights.core.spec_factory
          enabled: true
        - name: insights.specs.Specs.hostname
          enabled: true
        - name: insights.specs.default.DefaultSpecs.hostname
          enabled: true
        - name: insights.parsers.hostname.Hostname
          enabled: true
        - name: insights.tests.test_specs_runtime_ds_obfuscation.Specs
          enabled: true
        - name: insights.tests.test_specs_runtime_ds_obfuscation.Stuff
          enabled: true
""".strip()


class Specs(SpecSet):
    # datasource spec
    # - no need to filter
    # - must be obfuscated
    # - must be redacted
    ds_read_hostname = RegistryPoint(filterable=False, no_obfuscate=[], no_redact=False)


class Stuff(Specs):

    @datasource(Hostname, HostContext)
    def path_with_hostname(broker):
        hn = broker.get(Hostname).hostname
        # the hostname must be not obfuscated
        return '{0}{1}'.format(base_path, hn)

    ds_read_hostname = command_with_args(
            'ls -l  %s',
            path_with_hostname,
            save_as='localhost_empty'
    )


#
# TEST
#


def setup_function(func):
    open(tmp_file_path, 'w+').close()


def teardown_function(func):
    os.remove(tmp_file_path)
    # Reset Test ENV
    dr.COMPONENTS = defaultdict(lambda: defaultdict(set))
    dr.TYPE_OBSERVERS = defaultdict(set)
    dr.ENABLED = defaultdict(lambda: True)


@pytest.mark.parametrize("obfuscate", [True, False])
@patch('insights.core.spec_cleaner.Cleaner.generate_report')
def test_specs_ds_with_hn_collect(mock_fun, obfuscate):
    # Preparation
    manifest = collect.load_manifest(specs_manifest)
    for pkg in manifest.get("plugins", {}).get("packages", []):
        dr.load_components(pkg, exclude=None)
    # For verifying convenience, test obfuscate=False only
    conf = InsightsConfig(
            obfuscate=obfuscate, obfuscate_hostname=obfuscate,
            manifest=manifest)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()
    output_path, errors = collect.collect(
            tmp_path=arch.tmp_dir,
            archive_name=arch.archive_name,
            client_config=conf)
    meta_data_root = os.path.join(output_path, 'meta_data')
    data_root = os.path.join(output_path, 'data')

    assert not errors
    count = 0
    for spec in Specs.__dict__:
        if not spec.startswith(('__', 'context_handlers', 'registry')):
            file_name = "insights.tests.test_specs_runtime_ds_obfuscation.Specs.{0}.json".format(spec)
            meta_data = os.path.join(meta_data_root, file_name)
            with open(meta_data, 'r') as fp:
                mdata = json.load(fp)
                assert not mdata.get('errors')

                results = mdata.get('results')
                assert results

                if not isinstance(results, list):
                    results = [results]
                count += 1
                for result in results:
                    rel = result['object']['relative_path']
                    # save_as is used
                    assert result['object']['save_as'] is True
                    assert orig_hostname not in rel
                    assert rel.endswith(collected_file)
                    with open(os.path.join(data_root, rel), 'r') as fp:
                        new_content = ''.join(fp.readlines())
                        if obfuscate:
                            # hostname is obfuscated
                            assert orig_hostname not in new_content
                        else:
                            # hostname is not obfuscated
                            assert orig_hostname in new_content

    assert count == 1  # Number of Specs

    arch.delete_archive_dir()



import doctest

from insights.combiners import sap
from insights.combiners.hostname import Hostname
from insights.combiners.sap import Sap
from insights.parsers.hostname import Hostname as HnF
from insights.parsers.saphostctrl import SAPHostCtrlInstances
from insights.tests import context_wrap
from insights.tests.parsers.test_saphostctrl import SAPHOSTCTRL_HOSTINSTANCES_DOCS, SAPHOSTCTRL_HOSTINSTANCES_GOOD

HOSTNAME = 'lu0417.example.com'
HOSTNAME1 = 'li-ld-1810.example.com'
HOSTNAME2 = 'host_1.example.corp'
HOSTNAME3 = 'host_2.example.corp'

SAPHOSTCTRL_HOSTINSTANCES_R_CASE = '''
*********************************************************
 SID , String , R4D
 SystemNumber , String , 12
 InstanceName , String , DVEBMGS12
 InstanceType , String , Primary Application Server
 Hostname , String , host_1
 FullQualifiedHostname , String , host_1.example.corp
 SapVersionInfo , String , 753, patch 501, changelist 1967207
*********************************************************
 SID , String , R4D
 SystemNumber , String , 10
 InstanceName , String , ASCS10
 InstanceType , String , ABAP Central Services
 Hostname , String , host_1
 FullQualifiedHostname , String , host_1.example.corp
 SapVersionInfo , String , 753, patch 501, changelist 1967207
*********************************************************
 SID , String , WDX
 SystemNumber , String , 20
 InstanceName , String , W20
 InstanceType , String , WebDispatcher
 Hostname , String , host_2
 FullQualifiedHostname , String , host_2.example.corp
 SapVersionInfo , String , 773, patch 121, changelist 1917131
*********************************************************
 SID , String , SMD
 SystemNumber , String , 98
 InstanceName , String , SMDA98
 InstanceType , String , Solution Manager Diagnostic Agent
 Hostname , String , host_2
 FullQualifiedHostname , String , host_2.example.corp
 SapVersionInfo , String , 745, patch 400, changelist 1734487
*********************************************************
 SID , String , SMD
 SystemNumber , String , 97
 InstanceName , String , SMDA97
 InstanceType , String , Solution Manager Diagnostic Agent
 Hostname , String , host_2
 FullQualifiedHostname , String , host_2.example.corp
 SapVersionInfo , String , 745, patch 400, changelist 1734487
'''
SAPHOSTCTRL_HOSTINSTANCES_R_CASE_WO_TYPE = '''
*********************************************************
 SID , String , R4D
 SystemNumber , String , 12
 InstanceName , String , DVEBMGS12
 Hostname , String , host_1
 FullQualifiedHostname , String , host_1.example.corp
 SapVersionInfo , String , 753, patch 501, changelist 1967207
*********************************************************
 SID , String , R4D
 SystemNumber , String , 10
 InstanceName , String , ASCS10
 Hostname , String , host_1
 FullQualifiedHostname , String , host_1.example.corp
 SapVersionInfo , String , 753, patch 501, changelist 1967207
*********************************************************
 SID , String , WDX
 SystemNumber , String , 20
 InstanceName , String , W20
 Hostname , String , host_2
 FullQualifiedHostname , String , host_2.example.corp
 SapVersionInfo , String , 773, patch 121, changelist 1917131
*********************************************************
 SID , String , SMD
 SystemNumber , String , 98
 InstanceName , String , SMDA98
 Hostname , String , host_2
 FullQualifiedHostname , String , host_2.example.corp
 SapVersionInfo , String , 745, patch 400, changelist 1734487
*********************************************************
 SID , String , SMD
 SystemNumber , String , 97
 InstanceName , String , SMDA97
 Hostname , String , host_2
 FullQualifiedHostname , String , host_2.example.corp
 SapVersionInfo , String , 745, patch 400, changelist 1734487
'''


def test_saphostcrtl_hana():
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_DOCS))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None)
    sap = Sap(hn, inst)
    assert 'D50' not in sap
    assert 'HDB90' in sap.all_instances
    assert sap['HDB88'].number == '88'
    assert sap['HDB90'].hostname == 'lu0418'
    assert sap['HDB90'].version == '749, patch 211, changelist 1754007'
    assert sap.number('HDB90') == '90'
    assert sap.sid('HDB88') == 'D89'
    assert sap.is_netweaver is False
    assert sap.is_hana is True
    assert sap.is_ascs is False


def test_saphostcrtl_hana_2():
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_GOOD))
    hn = Hostname(HnF(context_wrap(HOSTNAME1)), None, None, None)
    sap = Sap(hn, inst)
    assert 'D50' not in sap
    assert 'HDB00' in sap
    assert sorted(sap.all_instances) == sorted([
        'ASCS07', 'ASCS52', 'D54', 'DVEBMGS09', 'ERS08', 'HDB00', 'HDB62',
        'HDB88', 'HDB90', 'SCS10', 'SMDA91'])
    assert sorted(sap.instances) == sorted([
        'ASCS07', 'ASCS52', 'D54', 'DVEBMGS09', 'ERS08', 'HDB00', 'HDB62',
        'HDB88', 'HDB90', 'SCS10'])
    assert sorted(sap.daa_instances) == sorted(['SMDA91'])
    assert sap['HDB88'].number == '88'
    assert sap['HDB90'].hostname == 'li-ld-1810'
    assert sap['DVEBMGS09'].version == '749, patch 301, changelist 1779613'
    assert sap.version('HDB90') == '749, patch 211, changelist 1754007'
    assert sap.hostname('HDB62') == 'd62dbsrv'
    assert sap.type('SCS10') == 'SCS'  # noqa E721
    assert sap.full_type('SCS10') == 'Java Central Services'
    assert sap.is_netweaver is True
    assert sap.is_hana is True
    assert sap.is_ascs is True


def test_r_case():
    saphostctrl = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_R_CASE))
    hn = Hostname(HnF(context_wrap(HOSTNAME3)), None, None, None)
    sap = Sap(hn, saphostctrl)
    assert sap['DVEBMGS12'].version == '753, patch 501, changelist 1967207'
    assert sap['ASCS10'].hostname == 'host_1'
    assert len(sap.daa_instances) == 2
    assert len(sap.instances) == 3
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is True


def test_r_case_wo_type():
    saphostctrl = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_R_CASE_WO_TYPE))
    hn = Hostname(HnF(context_wrap(HOSTNAME3)), None, None, None)
    sap = Sap(hn, saphostctrl)
    assert sap['DVEBMGS12'].version == '753, patch 501, changelist 1967207'
    assert sap['ASCS10'].hostname == 'host_1'
    assert len(sap.daa_instances) == 2
    assert len(sap.instances) == 3
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is True


def test_doc_examples():
    env = {
            'saps': Sap(
                Hostname(HnF(context_wrap(HOSTNAME3)), None, None, None),
                SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_R_CASE))
            )
          }
    failed, total = doctest.testmod(sap, globs=env)
    assert failed == 0



"""
SatelliteContentHostsCount - command ``psql -d foreman -c 'select count(*) from hosts'``
========================================================================================

The SatelliteContentHostsCount parser reads the output of
``psql -d foreman -c 'select count(*) from hosts'``.

Sample output of ``psql -d foreman -c 'select count(*) from hosts'``::

     count
    -------
        13
    (1 row)

Examples::

    >>> type(clients)
    <class 'insights.parsers.satellite_content_hosts_count.SatelliteContentHostsCount'>
    >>> clients.count
    13
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.satellite_content_hosts_count)
class SatelliteContentHostsCount(CommandParser):
    """
    Read the ``psql -d foreman -c 'select count(*) from hosts'``
    and set the hosts count to property ``count``.

    Attributes:
        count (int): The count of satellite content hosts
    """

    def parse_content(self, content):
        self.count = None
        if len(content) >= 3 and content[0].strip() == 'count':
            try:
                self.count = int(content[2].strip())
            except ValueError:
                raise ParseException("Unknow satelite content hosts count")
        if self.count is None:
            raise SkipComponent("Cannot get the count of satellite content hosts")



# coding=utf-8
import pytest

from insights.combiners.logrotate_conf import LogRotateConfTree
from insights.parsers import logrotate_conf
from insights.parsr.query import first
from insights.tests import context_wrap


CONF = """
# see "man logrotate" for details
# rotate log files weekly
weekly

# keep 4 weeks worth of backlogs
rotate 4

# create new (empty) log files after rotating old ones
create

# use date as a suffix of the rotated file
dateext

# uncomment this if you want your log files compressed
#compress

# RPM packages drop log rotation information into this directory
include /etc/logrotate.d

# no packages own wtmp and btmp -- we'll rotate them here
/var/log/wtmp {
    missingok
    monthly
    create 0664 root utmp
    minsize 1M
    rotate 1
    postrotate
        do some stuff to wtmp
    endscript
}

/var/log/btmp {
    missingok
    monthly
    create 0600 root utmp
    postrotate
        do some stuff to btmp
    endscript
    rotate 1
}

# system-specific logs may be also be configured here.
""".strip()


JUNK_SPACE = """
#SEG_15.06.01 
/var/log/spooler
{
    compress
    missingok
    rotate 30
    size 1M
    sharedscripts
    postrotate
        /bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}
""".strip()


LOGROTATE_MISSING_ENDSCRIPT = """
/var/log/example/*.log {
  daily
  missingok
  rotate 10
  dateext
  dateyesterday
  notifempty
  sharedscripts
  postrotate
    [ ! -f /var/run/openresty.pid ] || kill -USR1 `cat /var/run/example.pid`
    /usr/local/bin/mc cp $1 minio/matrix-prod-cluster/node1/example/
}
""".strip()

CONF_NO_INCLUDE = """
weekly
rotate 4
create
dateext

/var/log/wtmp {
    missingok
    monthly
    create 0664 root utmp
    minsize 1M
    rotate 1
    postrotate
        do some stuff to wtmp
    endscript
}

/var/log/btmp {
    missingok
    monthly
    create 0600 root utmp
    postrotate
        do some stuff to btmp
    endscript
    rotate 1
}

""".strip()


def test_logrotate_tree():
    p1 = logrotate_conf.LogRotateConfPEG(context_wrap(CONF, path="/etc/logrotate.conf"))
    p2 = logrotate_conf.LogRotateConfPEG(context_wrap(JUNK_SPACE, path="/etc/logrotate.d/test"))
    conf = LogRotateConfTree([p1, p2])
    assert 'rotate' in conf
    assert len(conf["weekly"]) == 1
    assert len(conf["/var/log/wtmp"]["missingok"]) == 1
    assert conf["/var/log/wtmp"]["postrotate"][first].value == "do some stuff to wtmp"

    assert len(conf["/var/log/btmp"]["rotate"]) == 1
    assert len(conf["/var/log/btmp"]["postrotate"]) == 1
    assert conf["/var/log/btmp"]["postrotate"][first].value == "do some stuff to btmp"

    assert 'compress' in conf["/var/log/spooler"]


def test_junk_space():
    p = logrotate_conf.LogRotateConfPEG(context_wrap(JUNK_SPACE, path="/etc/logrotate.conf"))
    conf = LogRotateConfTree([p])
    assert "compress" in conf["/var/log/spooler"]


def test_logrotate_conf_combiner_missing_endscript():
    with pytest.raises(Exception):
        logrotate_conf.LogRotateConfPEG(context_wrap(LOGROTATE_MISSING_ENDSCRIPT, path='/etc/logrotate.conf')),


def test_logrotate_conf_combiner_missing_include():
    p1 = logrotate_conf.LogRotateConfPEG(context_wrap(CONF_NO_INCLUDE, path="/etc/logrotate.conf"))
    p2 = logrotate_conf.LogRotateConfPEG(context_wrap(JUNK_SPACE, path="/etc/logrotate.d/test"))
    conf = LogRotateConfTree([p1, p2])
    assert 'rotate' in conf
    assert len(conf["weekly"]) == 1
    assert len(conf["/var/log/wtmp"]["missingok"]) == 1
    assert conf["/var/log/wtmp"]["postrotate"][first].value == "do some stuff to wtmp"
    assert '/var/log/spooler' not in conf



"""
CloudInit
=========

Parsers related to "cloud-init" can be placed in this module.

CloudInitQuery - cmd `cloud-init query -f '{{ cloud_name, platform }}'`
-----------------------------------------------------------------------
"""

from insights import CommandParser, parser
from insights.core.exceptions import SkipComponent
from insights.specs import Specs


@parser(Specs.cloud_init_query)
class CloudInitQuery(CommandParser):
    """
    Parse the ``cloud-init query -f '{{ cloud_name, platform }}'`` output.

    Sample input::

        ('azure', 'azure')

    Examples:
        >>> cloud_query.cloud_name == 'azure'
        True
        >>> cloud_query.platform == 'azure'
        True
    """

    def parse_content(self, content):
        try:
            if (
                len(content) == 1
                and content[0][0] == '('
                and content[0][-1] == ')'
                and ', ' in content[0]
            ):
                line = content[0].strip("()")
                self.cloud_name, self.platform = [
                    v.lstrip('u').strip('\'"') for v in line.split(', ')
                ]
            else:
                raise
        except Exception:
            raise SkipComponent("Invalid content: '{0}'".format(content))



"""
LsCPU - command ``lscpu``
=========================

This module provides the information about the CPU architecture using the output of the command ``lscpu``.
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.lscpu)
class LsCPU(CommandParser):
    """Parse the output of ``/usr/bin/lscpu``. It uses the ``CommandParser`` as the
    base class. The ``parse_content`` method also converts plural keys for
    better accessibility.

    Ex: "CPU(s)" is converted to "CPUs"

    Typical output of ``lscpu`` command is::

        Architecture:          x86_64
        CPU op-mode(s):        32-bit, 64-bit
        Byte Order:            Little Endian
        CPU(s):                2
        On-line CPU(s) list:   0,1
        Thread(s) per core:    2
        Core(s) per socket:    1
        Socket(s):             1
        NUMA node(s):          1
        Vendor ID:             GenuineIntel
        CPU family:            6
        Model:                 60
        Model name:            Intel Core Processor (Haswell, no TSX)
        Stepping:              1
        CPU MHz:               2793.530
        BogoMIPS:              5587.06
        Hypervisor vendor:     KVM
        Virtualization type:   full
        L1d cache:             32K
        L1i cache:             32K
        L2 cache:              4096K
        NUMA node0 CPU(s):     0,1

    Examples:

        >>> output.info['Architecture']
        'x86_64'
        >>> len(output.info)
        22
        >>> output.info['CPUs']
        '2'
        >>> output.info['Threads per core']
        '2'
        >>> output.info['Cores per socket']
        '1'
        >>> output.info['Sockets']
        '1'

    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("No data.")
        self.info = {}
        split_on = ":"
        for line in content:
            # On R9 they added indentation and section headers,
            # so make sure the split len is 2 otherwise skip.
            kv = line.split(split_on, 1)
            if len(kv) != 2:
                continue

            self.info[kv[0].replace("(s)", "s").strip()] = kv[1].strip()



# -*- coding: UTF-8 -*-
import os
import six

try:
    from unittest.mock import patch, Mock, mock_open
except Exception:
    from mock import patch, Mock, mock_open

from pytest import raises

from insights.specs.datasources.compliance import ComplianceClient

PATH = '/usr/share/xml/scap/ref_id.xml'


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_system_policies(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client._inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(
        return_value=Mock(status_code=200, json=Mock(return_value={'data': ['test']}))
    )
    assert compliance_client.get_system_policies() == ['test']
    url = "https://localhost/app/compliance/v2/systems/{0}/policies".format(
        compliance_client.inventory_id
    )
    compliance_client.conn.session.get.assert_called_with(url)


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_system_policies_error(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client._inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=500))
    assert compliance_client.get_system_policies() == []
    url = "https://localhost/app/compliance/v2/systems/{0}/policies".format(
        compliance_client.inventory_id
    )
    compliance_client.conn.session.get.assert_called_with(url)


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_profile_files(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.os_major = '7'
    assert compliance_client.profile_files() == []


@patch("insights.specs.datasources.compliance.call", return_value=(0, PATH))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy(config, call):
    compliance_client = ComplianceClient(config=config)
    compliance_client.profile_files = lambda: ['/something']
    assert compliance_client.find_scap_policy('ref_id') == PATH


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy_with_one_datastream_file(config, tmpdir):
    compliance_client = ComplianceClient(config=config)
    dir1 = tmpdir.mkdir('scap')
    file = dir1.join('test_file.xml')
    file.write(
        """
    <xccdf-1.2:Profile id="xccdf_org.ssgproject.content_profile_anssi_bp28_high">
    </xccdf-1.2:Profile>
        """
    )
    compliance_client.profile_files = lambda: [str(file)]
    with patch("insights.specs.datasources.compliance.SCAP_DATASTREAMS_PATH", str(dir1) + "/"):
        assert compliance_client.find_scap_policy('content_profile_anssi_bp28_high') == file


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy_with_two_datastream_file(config, tmpdir):
    compliance_client = ComplianceClient(config=config)
    dir1 = tmpdir.mkdir('scap')
    file1 = dir1.join('test_file1.xml')
    file1.write(
        """
    <xccdf-1.2:Profile id="xccdf_org.ssgproject.content_profile_anssi_bp28_high">
    </xccdf-1.2:Profile>
        """
    )
    file2 = dir1.join('test_file2.xml')
    file2.write(
        """
    <xccdf-1.2:Profile id="xccdf_org.ssgproject.content_profile_anssi_bp28_high">
    </xccdf-1.2:Profile>
        """
    )
    compliance_client.profile_files = lambda: [str(file1), str(file2)]
    with patch("insights.specs.datasources.compliance.SCAP_DATASTREAMS_PATH", str(dir1) + "/"):
        assert compliance_client.find_scap_policy('content_profile_anssi_bp28_high') == file1


@patch(
    "insights.specs.datasources.compliance.call",
    return_value=(1, 'bad things happened'.encode('utf-8')),
)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy_not_found(config, call):
    compliance_client = ComplianceClient(config=config)
    compliance_client.profile_files = lambda: ['/something']
    assert compliance_client.find_scap_policy('ref_id') is None


@patch("insights.specs.datasources.compliance.call", return_value=(0, ''.encode('utf-8')))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_run_scan(config, call):
    compliance_client = ComplianceClient(config=config)
    output_path = '/tmp/oscap_results-ref_id.xml'
    env = os.environ
    env.update({'TZ': 'UTC'})
    compliance_client.run_scan('ref_id', '/nonexistent', output_path)
    if six.PY3:
        call.assert_called_with(
            ("oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent'),
            keep_rc=True,
            env=env,
        )
    else:
        call.assert_called_with(
            (
                "oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent'
            ).encode(),
            keep_rc=True,
            env=env,
        )


@patch(
    "insights.specs.datasources.compliance.call",
    return_value=(1, 'bad things happened'.encode('utf-8')),
)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_run_scan_fail(config, call):
    compliance_client = ComplianceClient(config=config)
    output_path = '/tmp/oscap_results-ref_id.xml'
    env = os.environ
    env.update({'TZ': 'UTC'})
    with raises(SystemExit):
        compliance_client.run_scan('ref_id', '/nonexistent', output_path)
    if six.PY3:
        call.assert_called_with(
            ("oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent'),
            keep_rc=True,
            env=env,
        )
    else:
        call.assert_called_with(
            (
                "oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent'
            ).encode(),
            keep_rc=True,
            env=env,
        )


@patch("insights.specs.datasources.compliance.call", return_value=(0, ''.encode('utf-8')))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_run_scan_missing_profile(config, call):
    compliance_client = ComplianceClient(config=config)
    output_path = '/tmp/oscap_results-ref_id.xml'
    env = os.environ
    env.update({'TZ': 'UTC'})
    assert compliance_client.run_scan('ref_id', None, output_path) is None
    call.assert_not_called()


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_not_downloaded_if_not_needed(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=204))
    assert compliance_client.download_tailoring_file({'id': 'foo', 'ref_id': 'aaaaa'}) is None


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_not_downloaded_if_os_minor_version_mismatches(config):
    compliance_client = ComplianceClient(os_version=['6', '5'], config=config)
    compliance_client.conn.session.get = Mock(
        return_value=Mock(
            status_code=200,
            headers={"Content-Type": "application/xml"},
            json=Mock(return_value={'data': []}),
        )
    )
    assert (
        compliance_client.download_tailoring_file(
            {'id': 'foo', 'ref_id': 'aaaaa', 'os_minor_version': '2'}
        )
        is None
    )


@patch("insights.specs.datasources.compliance.open", new_callable=mock_open)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_downloaded_if_needed(config, call):
    compliance_client = ComplianceClient(os_version=['6', '5'], config=config)
    compliance_client.conn.session.get = Mock(
        return_value=Mock(
            status_code=200,
            headers={"Content-Type": "application/xml"},
            json=Mock(return_value={'data': []}),
        )
    )
    assert 'oscap_tailoring_file-aaaaa' in compliance_client.download_tailoring_file(
        {'id': 'foo', 'ref_id': 'aaaaa', 'os_minor_version': '5'}
    )


@patch("insights.specs.datasources.compliance.open", new_callable=mock_open)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_fails_to_download(config, call):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn.session.get = Mock(
        return_value=Mock(
            status_code=403,
            ok=False,
            headers={"Content-Type": "application/xml"},
            json=Mock(return_value={'data': []}),
        )
    )
    assert compliance_client.download_tailoring_file({'id': 'foo', 'ref_id': 'aaaaa'}) is None

    compliance_client.conn.session.get = Mock(
        return_value=Mock(
            status_code=200,
            headers={"Content-Type": "application/json"},
            json=Mock(return_value={'data': []}),
        )
    )
    assert compliance_client.download_tailoring_file({'id': 'foo', 'ref_id': 'aaaaa'}) is None

    compliance_client.conn.session.get = Mock(
        return_value=Mock(
            status_code=200,
            content=None,
            headers={"Content-Type": "application/xml"},
            json=Mock(return_value={'data': []}),
        )
    )
    assert compliance_client.download_tailoring_file({'id': 'foo', 'ref_id': 'aaaaa'}) is None


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_build_oscap_command_does_not_append_tailoring_path(config):
    compliance_client = ComplianceClient(config=config)
    expected_command = 'oscap xccdf eval --profile aaaaa --results output_path xml_sample'
    assert expected_command == compliance_client.build_oscap_command(
        'aaaaa', 'xml_sample', 'output_path', None
    )


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_build_oscap_command_append_tailoring_path(config):
    compliance_client = ComplianceClient(config=config)
    expected_command = 'oscap xccdf eval --profile aaaaa --tailoring-file tailoring_path --results output_path xml_sample'
    assert expected_command == compliance_client.build_oscap_command(
        'aaaaa', 'xml_sample', 'output_path', 'tailoring_path'
    )


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test__inventory_id(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn._fetch_system_by_machine_id = lambda: []
    with raises(SystemExit):
        compliance_client.inventory_id

    compliance_client.conn._fetch_system_by_machine_id = lambda: [{}]
    with raises(SystemExit):
        compliance_client.inventory_id

    compliance_client.conn._fetch_system_by_machine_id = lambda: [{'id': '12345'}]
    assert compliance_client.inventory_id == '12345'


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_assignable_policies(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn.session.get = Mock(
        return_value=Mock(status_code=200, json=Mock(return_value={'data': ['test']}))
    )
    compliance_client._inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.get_system_policies = lambda: []
    compliance_client.os_major = 9
    compliance_client.os_minor = 3
    compliance_client.conn.session.get = Mock(
        return_value=Mock(
            status_code=200, json=Mock(return_value={'data': [{"id": 123456, "title": "foo"}]})
        )
    )
    assert compliance_client.assignable_policies() == 0
    url = "https://localhost/app/compliance/v2/policies?filter=(os_major_version=9 and os_minor_version=3)"
    compliance_client.conn.session.get.assert_called_with(url)


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_policy_link_assign(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client._inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    policy_id = "d83ddbac-ab56-420b-9e71-878795375af5"
    compliance_client.conn.session.patch = Mock(
        return_value=Mock(status_code=202, json=Mock(return_value={}))
    )
    assert compliance_client.policy_link(policy_id, 'patch') == 0
    url = "https://localhost/app/compliance/v2/policies/{0}/systems/{1}".format(
        policy_id, compliance_client.inventory_id
    )
    compliance_client.conn.session.patch.assert_called_with(url)


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_policy_link_unassign(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn.session.delete = Mock(
        return_value=Mock(status_code=202, json=Mock(return_value={}))
    )
    compliance_client._inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    policy_id = "d83ddbac-ab56-420b-9e71-878795375af5"
    assert compliance_client.policy_link(policy_id, 'delete') == 0
    url = "https://localhost/app/compliance/v2/policies/{0}/systems/{1}".format(
        policy_id, compliance_client.inventory_id
    )
    compliance_client.conn.session.delete.assert_called_with(url)



from collections import defaultdict

from insights.core import filters
from insights.specs import Specs
from insights.tests import context_wrap, DEFAULT_RELEASE, DEFAULT_HOSTNAME

DATA = """
One
Two
Three
Four
"""
DATA_FILTERED = """
Two
Four
"""


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


def test_context_wrap_unfiltered():
    context = context_wrap(DATA)
    assert context is not None
    assert context.content == DATA.strip().splitlines()
    assert context.release == DEFAULT_RELEASE
    assert context.hostname == DEFAULT_HOSTNAME
    assert context.version == ["-1", "-1"]
    assert context.machine_id == "machine_id"


def test_context_wrap_filtered():
    filters.add_filter(Specs.messages, ["Two", "Four", "Five"])
    context = context_wrap(DATA, filtered_spec=Specs.messages)
    assert context is not None
    assert context.content == DATA_FILTERED.strip().splitlines()



import logging
import os

from insights.core import archives, dr
from insights.core.context import (ClusterArchiveContext, ExecutionContextMeta, HostArchiveContext,
                                   SerializedArchiveContext)
from insights.core.exceptions import InvalidArchive
from insights.core.serde import Hydration

log = logging.getLogger(__name__)

if hasattr(os, "scandir"):
    def get_all_files(path):
        with os.scandir(path) as it:
            for ent in it:
                try:
                    if ent.is_dir(follow_symlinks=False):
                        for pth in get_all_files(ent.path):
                            yield pth
                    elif ent.is_file(follow_symlinks=False):
                        yield ent.path
                except OSError as ex:
                    log.exception(ex)


else:
    def get_all_files(path):
        for root, _, files in os.walk(path):
            for f in files:
                full_path = os.path.join(root, f)
                if os.path.isfile(full_path) and not os.path.islink(full_path):
                    yield full_path


def identify(files):
    common_path, ctx = ExecutionContextMeta.identify(files)
    if ctx:
        return common_path, ctx

    common_path = os.path.dirname(os.path.commonprefix(files))
    if not common_path:
        raise InvalidArchive("Unable to determine common path")

    return common_path, HostArchiveContext


def create_context(path, context=None):
    top = os.listdir(path)
    arc = [os.path.join(path, f) for f in top
           if f.endswith(archives.COMPRESSION_TYPES) and
           os.path.isfile(os.path.join(path, f))]
    if arc:
        return ClusterArchiveContext(path, all_files=arc)

    all_files = list(get_all_files(path))
    if not all_files:
        raise InvalidArchive("No files in archive")

    common_path, ctx = identify(all_files)
    context = context or ctx
    return context(common_path, all_files=all_files)


def initialize_broker(path, context=None, broker=None):
    ctx = create_context(path, context=context)
    broker = broker or dr.Broker()
    if isinstance(ctx, ClusterArchiveContext):
        return ctx, broker

    broker[ctx.__class__] = ctx
    if isinstance(ctx, SerializedArchiveContext):
        h = Hydration(root=ctx.root, ctx=ctx)
        broker = h.hydrate(broker=broker)
    return ctx, broker



import doctest

from collections import defaultdict

from insights.core import filters
from insights.parsers import messages
from insights.specs import Specs
from insights.tests import context_wrap

MSGINFO = """
May 18 15:13:34 lxc-rhel68-sat56 jabberd/sm[11057]: session started: jid=rhn-dispatcher-sat@lxc-rhel6-sat56.redhat.com/superclient
May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon
May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM...
May 18 15:24:28 lxc-rhel68-sat56 yum[11597]: Installed: lynx-2.8.6-27.el6.x86_64
May 18 15:36:19 lxc-rhel68-sat56 yum[11954]: Updated: sos-3.2-40.el6.noarch
Apr 22 10:35:01 boy-bona CROND[27921]: (root) CMD (/usr/lib64/sa/sa1 -S DISK 1 1)
Apr 22 10:37:32 boy-bona crontab[28951]: (root) LIST (root)
Apr 22 10:40:01 boy-bona CROND[30677]: (root) CMD (/usr/lib64/sa/sa1 -S DISK 1 1)
Apr 22 10:41:13 boy-bona crontab[32515]: (root) LIST (root)
""".strip()

filters.add_filter(Specs.messages, [
    "LIST",
    "CROND",
    "jabberd",
    "Wrapper",
    "Launching",
    "yum"
])


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


def test_doc_examples():
    env = {
        'msgs': messages.Messages(context_wrap(MSGINFO)),
        'Messages': messages.Messages
    }
    failed, total = doctest.testmod(messages, globs=env)
    assert failed == 0


def test_messages():
    msg_info = messages.Messages(context_wrap(MSGINFO))
    bona_list = msg_info.get('(root) LIST (root)')
    assert 2 == len(bona_list)
    assert bona_list[0].get('timestamp') == "Apr 22 10:37:32"
    assert bona_list[1].get('timestamp') == "Apr 22 10:41:13"
    crond = msg_info.get('CROND')
    assert 2 == len(crond)
    assert crond[0].get('procname') == "CROND[27921]"
    assert msg_info.get('jabberd/sm[11057]')[0].get('hostname') == "lxc-rhel68-sat56"
    assert msg_info.get('Wrapper')[0].get('message') == "--> Wrapper Started as Daemon"
    assert msg_info.get('Launching')[0].get('raw_message') == "May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM..."
    assert 2 == len(msg_info.get('yum'))



import logging

import six
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel.yaml.comments import (
    CommentedMap,
    CommentedSeq,
)

logger = logging.getLogger(__name__)


class PlaybookSerializer:
    @classmethod
    def serialize(cls, value):
        """Serialize a playbook into a string.

        :returns: Serialized playbook.
        :rtype: str
        """
        return cls._obj(value)

    @classmethod
    def _obj(cls, value):
        """
        :type value: any
        :rtype: str
        """
        if isinstance(value, dict) or isinstance(value, CommentedMap):
            return cls._dict(value)
        if isinstance(value, list) or isinstance(value, CommentedSeq):
            return cls._list(value)
        if isinstance(value, int) or isinstance(value, float):
            return str(value)
        if isinstance(value, six.string_types):
            return cls._str(value)

        logger.debug("Value type not recognized, it may misbehave: {value} ({typ})".format(
            value=value, typ=type(value).__name__)
        )
        return "'" + str(value) + "'"

    @classmethod
    def _str(cls, value):
        """
        :type value: str
        :rtype: str
        """
        # no quote      'no quote'
        # single'quote  "single'quote"
        # double"quote  'double"quote'
        # both"'quotes  'both"\'quotes'

        quote = "'"
        if "'" in value:
            if '"' not in value:
                quote = '"'
            else:
                value = value.replace("'", "\\'")

        return quote + value + quote

    @classmethod
    def _dict(cls, value):
        """
        :type value: dict | yaml.comments.CommentedMap
        :rtype: str
        """
        result = "ordereddict(["
        result += ", ".join(
            "('{key}', {value})".format(key=k, value=cls._obj(v))
            for k, v in value.items()
        )
        result += "])"
        return result

    @classmethod
    def _list(cls, value):
        """
        :type value: list | yaml.comments.CommentedSeq
        :rtype: str
        """
        result = "["
        result += ", ".join(cls._obj(v) for v in value)
        result += "]"
        return result



#!/usr/bin/env python

from __future__ import print_function

from insights import rule, run, make_metadata, add_filter
from insights.combiners.cloud_instance import CloudInstance
from insights.combiners.cloud_provider import CloudProvider
from insights.core.dr import set_enabled, load_components
from insights.parsers.aws_instance_id import AWSInstanceIdDoc
from insights.parsers.azure_instance import AzureInstanceID, AzureInstanceType
from insights.parsers.client_metadata import MachineID
from insights.parsers.dmidecode import DMIDecode
from insights.parsers.etc_machine_id import EtcMachineId
from insights.parsers.gcp_instance_type import GCPInstanceType
from insights.parsers.hostname import Hostname
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.ip import IPs
from insights.parsers.mac import MacAddress
from insights.parsers.rhsm_conf import RHSMConf
from insights.parsers.subscription_manager import SubscriptionManagerID, SubscriptionManagerFacts
from insights.parsers.yum import YumRepoList


add_filter(SubscriptionManagerFacts, 'instance_id')
add_filter(RHSMConf, ['server', 'hostname'])


def _filter_falsy(dict_):
    return dict((k, v) for k, v in dict_.items() if v)


@rule(
    optional=[
        MachineID,
        EtcMachineId,
        DMIDecode,
        SubscriptionManagerID,
        IPs,
        Hostname,
        MacAddress,
        CloudInstance,
    ]
)
def canonical_facts(
    insights_id, machine_id, dmidecode, submanid, ips, hostname, mac_addresses,
    cloud_instance,
):

    facts = dict(
        insights_id=insights_id.uuid if insights_id else None,
        machine_id=machine_id.uuid if machine_id else None,
        bios_uuid=dmidecode.system_uuid if dmidecode else None,
        subscription_manager_id=submanid.uuid if submanid else None,
        ip_addresses=ips.ipv4_addresses if ips else [],
        mac_addresses=list(filter(None, [mc.address for mc in mac_addresses or []])),
        fqdn=hostname.fqdn if hostname else None,
        provider_id=cloud_instance.id if cloud_instance else None,
        provider_type=cloud_instance.provider if cloud_instance else None,
    )

    return make_metadata(**_filter_falsy(facts))


def get_canonical_facts(path=None):
    load_components("insights.specs.default", "insights.specs.insights_archive")

    required_components = [
        AWSInstanceIdDoc,
        AzureInstanceID,
        AzureInstanceType,
        CloudInstance,
        CloudProvider,
        DMIDecode,
        EtcMachineId,
        GCPInstanceType,
        Hostname,
        IPs,
        InstalledRpms,
        MacAddress,
        MachineID,
        RHSMConf,
        SubscriptionManagerFacts,
        SubscriptionManagerID,
        YumRepoList,
        canonical_facts,
    ]
    for comp in required_components:
        set_enabled(comp, True)

    br = run(canonical_facts, root=path)
    d = br[canonical_facts]
    del d["type"]
    return d


if __name__ == "__main__":
    import json

    print(json.dumps(get_canonical_facts()))



import json
import pytest

from collections import defaultdict, OrderedDict
from mock.mock import Mock

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs import Specs
from insights.specs.datasources.awx_manage import LocalSpecs, awx_manage_check_license_data_datasource


AWX_MANAGE_LICENSE = """
{"contact_email": "test@redhat.com", "company_name": "test Inc", "instance_count": 100, "license_date": 1655092799, "license_type": "enterprise", "subscription_name": "Red Hat Ansible Automation, Standard (100 Managed Nodes)", "sku": "MCT3691", "support_level": "Standard", "product_name": "Red Hat Ansible Automation Platform", "valid_key": true, "satellite": null, "pool_id": "2c92808179803e530179ea5989a157a4", "current_instances": 1, "available_instances": 100, "free_instances": 99, "time_remaining": 29885220, "trial": false, "grace_period_remaining": 32477220, "compliant": true, "date_warning": false, "date_expired": false}
""".strip()

NG_COMMAND = """
awx-manage: command not found
""".strip()

AWX_MANAGE_FILTER_JSON = {
    "license_type": "enterprise",
    "time_remaining": 29885220,
    "instance_count": 100,
    "support_level": "Standard"
}

RELATIVE_PATH = 'insights_commands/awx-manage_check_license_--data'


def setup_function(func):
    if func is test_ansible_tower_license_datasource or func is test_ansible_tower_license_datasource_NG_output:
        filters.add_filter(Specs.awx_manage_check_license_data, ["license_type", "support_level", "instance_count", "time_remaining"])
    if func is test_ansible_tower_license_datasource_no_filter:
        filters.add_filter(Specs.awx_manage_check_license_data, [])


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(set)


def test_ansible_tower_license_datasource():
    awx_manage_data = Mock()
    awx_manage_data.content = AWX_MANAGE_LICENSE.splitlines()
    broker = {LocalSpecs.awx_manage_check_license_data_raw: awx_manage_data}
    result = awx_manage_check_license_data_datasource(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=json.dumps(OrderedDict(sorted(AWX_MANAGE_FILTER_JSON.items()))), relative_path=RELATIVE_PATH)
    assert result.content == expected.content
    assert result.relative_path == expected.relative_path


def test_ansible_tower_license_datasource_no_filter():
    awx_manage_data = Mock()
    awx_manage_data.content = AWX_MANAGE_LICENSE.splitlines()
    broker = {LocalSpecs.awx_manage_check_license_data_raw: awx_manage_data}
    with pytest.raises(SkipComponent) as e:
        awx_manage_check_license_data_datasource(broker)
    assert 'SkipComponent' in str(e)


def test_ansible_tower_license_datasource_NG_output():
    awx_manage_data = Mock()
    awx_manage_data.content = NG_COMMAND.splitlines()
    broker = {LocalSpecs.awx_manage_check_license_data_raw: awx_manage_data}
    with pytest.raises(SkipComponent) as e:
        awx_manage_check_license_data_datasource(broker)
    assert 'Unexpected exception' in str(e)



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import cloud_init
from insights.tests import context_wrap

CLOUD_INIT_QUERY_OUTPUT1 = """
(u'azure', u'azure')
""".strip()

CLOUD_INIT_QUERY_OUTPUT2 = """
('aws', 'ec2')
""".strip()

CLOUD_INIT_QUERY_OUTPUT3 = """
2024-11-19 03:14:38,741 - query.py[WARNING]: Missing root-readable /run/cloud-init/instance-data-sensitive.json. Using redacted /run/cloud-init/instance-data.json instead.
2024-11-19 03:14:38,741 - query.py[ERROR]: Missing instance-data file: /run/cloud-init/instance-data.json
""".strip()

CLOUD_INIT_QUERY_OUTPUT4 = """
(2024-11-19, test, test)
""".strip()


def test_cloud_init_query():
    ret = cloud_init.CloudInitQuery(context_wrap(CLOUD_INIT_QUERY_OUTPUT1))
    assert ret.cloud_name == 'azure'
    assert ret.platform == 'azure'

    ret = cloud_init.CloudInitQuery(context_wrap(CLOUD_INIT_QUERY_OUTPUT2))
    assert ret.cloud_name == 'aws'
    assert ret.platform == 'ec2'

    with pytest.raises(SkipComponent):
        cloud_init.CloudInitQuery(context_wrap(CLOUD_INIT_QUERY_OUTPUT3))

    with pytest.raises(SkipComponent):
        cloud_init.CloudInitQuery(context_wrap(CLOUD_INIT_QUERY_OUTPUT4))


def test_cloud_init_docs():
    env = {
        'cloud_query': cloud_init.CloudInitQuery(context_wrap(CLOUD_INIT_QUERY_OUTPUT1)),
    }
    failed, total = doctest.testmod(cloud_init, globs=env)
    assert failed == 0



import pytest
import doctest
from insights.parsers.hostname import Hostname as HnF, HostnameShort as HnS, HostnameDefault as HnD
from insights.parsers.systemid import SystemID
from insights.combiners.hostname import Hostname
from insights.combiners import hostname
from insights.tests import context_wrap

HOSTNAME_FULL = "rhel7.example.com"
HOSTNAME_SHORT = "rhel7"
HOSTNAME_DEF = "rhel7"
SYSTEMID_PROFILE_NAME = '''
<?xml version="1.0"?>
<params>
<param>
<value><struct>
<member>
<name>username</name>
<value><string>testuser</string></value>
</member>
<member>
<name>profile_name</name>
<value><string>example_profile</string></value>
</member>
<member>
<name>system_id</name>
<value><string>ID-example</string></value>
</member>
<member>
<name>architecture</name>
<value><string>x86_64</string></value>
</member>
</struct></value>
</param>
</params>
'''.strip()
SYSTEMID_NO_PROFILE_NAME = '''
<?xml version="1.0"?>
<params>
<param>
<member>
<name>username</name>
<value><string>testuser</string></value>
</member>
<member>
<name>architecture</name>
<value><string>x86_64</string></value>
</member>
</param>
</params>
'''.strip()


def test_get_hostname():
    hnf = HnF(context_wrap(HOSTNAME_FULL))
    expected = (HOSTNAME_FULL, HOSTNAME_SHORT, 'example.com')
    result = Hostname(hnf, None, None, None)
    assert result.fqdn == expected[0]
    assert result.hostname == expected[1]
    assert result.domain == expected[2]

    hns = HnS(context_wrap(HOSTNAME_SHORT))
    expected = (HOSTNAME_SHORT, HOSTNAME_SHORT, '')
    result = Hostname(None, None, hns, None)
    assert result.fqdn == expected[0]
    assert result.hostname == expected[1]
    assert result.domain == expected[2]

    hnd = HnD(context_wrap(HOSTNAME_DEF))
    expected = (HOSTNAME_DEF, HOSTNAME_DEF, '')
    result = Hostname(None, hnd, None, None)
    assert result.fqdn == expected[0]
    assert result.hostname == expected[1]
    assert result.domain == expected[2]


def test_get_systemid_hostname():
    sid = SystemID(context_wrap(SYSTEMID_PROFILE_NAME))
    expected = ('example_profile', 'example_profile', '')
    result = Hostname(None, None, None, sid)
    assert result.fqdn == expected[0]
    assert result.hostname == expected[1]
    assert result.domain == expected[2]


def test_get_all_hostname():
    hnf = HnF(context_wrap(HOSTNAME_FULL))
    hns = HnS(context_wrap(HOSTNAME_SHORT))
    hnd = HnD(context_wrap(HOSTNAME_DEF))
    sid = SystemID(context_wrap(SYSTEMID_PROFILE_NAME))
    expected = (HOSTNAME_FULL, HOSTNAME_SHORT, 'example.com')
    result = Hostname(hnf, hnd, hns, sid)
    assert result.fqdn == expected[0]
    assert result.hostname == expected[1]
    assert result.domain == expected[2]


def test_hostname_raise():
    with pytest.raises(Exception):
        sid = SystemID(context_wrap(SYSTEMID_NO_PROFILE_NAME))
        Hostname(None, None, None, None, sid)


def test_hostname_doc():
    hnf = HnF(context_wrap(HOSTNAME_FULL))
    hns = HnS(context_wrap(HOSTNAME_SHORT))
    hnd = HnD(context_wrap(HOSTNAME_DEF))
    sid = SystemID(context_wrap(SYSTEMID_PROFILE_NAME))
    env = {
        'hostname': Hostname(hnf, hnd, hns, sid),
    }
    failed, total = doctest.testmod(hostname, globs=env)
    assert failed == 0



"""
Custom datasource to get the list of the rsyslog errorfile paths.
"""

from insights.core.context import HostContext
from insights.core.plugins import datasource
from insights.combiners.rsyslog_confs import RsyslogAllConf
from insights.core.exceptions import SkipComponent


@datasource(RsyslogAllConf, HostContext)
def rsyslog_errorfile(broker):
    """
    This datasource provides the list of the rsyslog errorfile paths.

    Sample data returned::

        '/var/log/omelasticsearch.log /var/log/oversized.log'

    Returns:
        string: string of the errorfile paths that separated by a whitespace.

    Raises:
        SkipComponent: When 'errorfile' is not configured.
    """

    rsyslog_confs = broker[RsyslogAllConf]
    if rsyslog_confs:
        result_list = []
        for conf_path in rsyslog_confs:
            for line in rsyslog_confs[conf_path]:
                if 'errorfile' in line and 'action' in line:
                    paths = line.split('errorfile')
                    for i in range(1, len(paths)):
                        errorfile_path = paths[i].split('"')[1]
                        result_list.append(errorfile_path)
        if result_list:
            return ' '.join(result_list)

    raise SkipComponent



import doctest

from insights.parsers import ssh
from insights.parsers.ssh import SshDConfig
from insights.parsers.ssh import SshDConfigD
from insights.parsers.ssh import SshdTestMode
from insights.tests import context_wrap


SSHD_CONFIG_INPUT = """
#	$OpenBSD: sshd_config,v 1.93 2014/01/10 05:59:19 djm Exp $

Port 22
#AddressFamily any
ListenAddress 10.110.0.1
Port 22
ListenAddress 10.110.1.1
#ListenAddress ::
# AllowUsers
AllowUsers
# The default requires explicit activation of protocol 1
#Protocol 2
Protocol 1
""".strip()


def test_sshd_config():
    sshd_config = SshDConfig(context_wrap(SSHD_CONFIG_INPUT))
    assert sshd_config is not None
    assert 'Port' in sshd_config
    assert 'PORT' in sshd_config
    assert sshd_config['port'] == ['22', '22']
    assert 'ListenAddress' in sshd_config
    assert sshd_config['ListenAddress'] == ['10.110.0.1', '10.110.1.1']
    assert sshd_config.last('ListenAddress') == '10.110.1.1'
    assert sshd_config['Protocol'] == ['1']
    assert 'AddressFamily' not in sshd_config
    ports = [l for l in sshd_config if l.keyword == 'Port']
    assert len(ports) == 2
    assert ports[0].value == '22'
    assert sshd_config.last('ListenAddress') == '10.110.1.1'


SSHD_CONFIG_COMPLETE = """
# Standard /etc/ssh/sshd_config without comments and blank lines
HostKey /etc/ssh/ssh_host_rsa_key
HostKey /etc/ssh/ssh_host_ecdsa_key
HostKey /etc/ssh/ssh_host_ed25519_key
SyslogFacility AUTHPRIV
AuthorizedKeysFile	.ssh/authorized_keys
PasswordAuthentication=yes
ChallengeResponseAuthentication no
GSSAPIAuthentication yes
GSSAPICleanupCredentials no
UsePAM yes
X11Forwarding = "yes"
UsePrivilegeSeparation sandbox		# Default for new installations.
AcceptEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
AcceptEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
AcceptEnv LC_IDENTIFICATION LC_ALL LANGUAGE
AcceptEnv XMODIFIERS
Subsystem	sftp	/usr/libexec/openssh/sftp-server
APlusOption only,the,last,shall,be,taken,in,account,by,does_line_user_plus()
APlusOption +val1,val2,val3
"""


def test_sshd_config_complete():
    config = SshDConfig(context_wrap(SSHD_CONFIG_COMPLETE))
    assert config is not None
    assert 'HostKey' in config
    hostkeys = config.get('HostKey')
    assert len(hostkeys) == 3
    for hostkey in hostkeys:
        assert hostkey.keyword == 'HostKey'
        assert hostkey.kw_lower == 'hostkey'
    assert hostkeys[0].value == '/etc/ssh/ssh_host_rsa_key'
    assert hostkeys[1].value == '/etc/ssh/ssh_host_ecdsa_key'
    assert hostkeys[2].value == '/etc/ssh/ssh_host_ed25519_key'
    assert hostkeys[0].line == 'HostKey /etc/ssh/ssh_host_rsa_key'
    assert hostkeys[1].line == 'HostKey /etc/ssh/ssh_host_ecdsa_key'
    assert hostkeys[2].line == 'HostKey /etc/ssh/ssh_host_ed25519_key'
    assert config['HostKey'] == [
        '/etc/ssh/ssh_host_rsa_key',
        '/etc/ssh/ssh_host_ecdsa_key',
        '/etc/ssh/ssh_host_ed25519_key'
    ]
    assert config.last('UsePAM') == 'yes'
    assert config.last('ClientAliveInterval') is None
    assert config.last('ClientAliveCountMax', '0') == '0'
    assert config.get_line('UsePAM') == 'UsePAM yes'
    assert config['PasswordAuthentication'] == ['yes']
    assert config['X11Forwarding'] == ['yes']
    assert config.get_line('ClientAliveInterval') == 'ClientAliveInterval   # Implicit default'
    assert config.get_line('ClientAliveCountMax', '0') == 'ClientAliveCountMax 0  # Implicit default'
    assert config.get_values('SyslogFacility') == ['AUTHPRIV']
    assert config.get_values('SyslogFacility', join_with='') == 'AUTHPRIV'
    assert config.get_values('ClientAliveInterval', default='') == ['']
    assert config.get_values('HostKey', default='no_key', join_with=' ') == \
        '/etc/ssh/ssh_host_rsa_key /etc/ssh/ssh_host_ecdsa_key /etc/ssh/ssh_host_ed25519_key'
    # Splitting does not occur if joining has not occurred:
    assert config.get_values('HostKey', default='no_key', split_on='/') == [
        '/etc/ssh/ssh_host_rsa_key',
        '/etc/ssh/ssh_host_ecdsa_key',
        '/etc/ssh/ssh_host_ed25519_key'
    ]
    assert config.get_values('AcceptEnv', join_with=' ', split_on=' ') == [
        'LANG', 'LC_CTYPE', 'LC_NUMERIC', 'LC_TIME', 'LC_COLLATE',
        'LC_MONETARY', 'LC_MESSAGES', 'LC_PAPER', 'LC_NAME', 'LC_ADDRESS',
        'LC_TELEPHONE', 'LC_MEASUREMENT', 'LC_IDENTIFICATION', 'LC_ALL',
        'LANGUAGE', 'XMODIFIERS'
    ]
    assert config.line_uses_plus("APlusOption") is True
    assert config.line_uses_plus("Subsystem") is False
    assert config.line_uses_plus("AcceptEnv") is False
    assert config.line_uses_plus("NonExistingOption") is None


SSHD_TEST_MODE = """
port 22
addressfamily any
listenaddress [::]:22
listenaddress 0.0.0.0:22
usepam yes
logingracetime 120
x11displayoffset 10
x11maxdisplays 1000
maxauthtries 6
ciphers aes256-gcm@openssh.com,chacha20-poly1305@openssh.com,aes256-ctr,aes128-gcm@openssh.com,aes128-ctr
macs hmac-sha2-256-etm@openssh.com,hmac-sha1-etm@openssh.com,umac-128-etm@openssh.com,hmac-sha2-512-etm@openssh.com,hmac-sha2-256,hmac-sha1,umac-128@openssh.com,hmac-sha2-512
""".strip()


def test_sshd_test_mode():
    sshd_mode = SshdTestMode(context_wrap(SSHD_TEST_MODE))
    assert len(sshd_mode) == 10
    assert sshd_mode.get("listenaddress") == ["[::]:22", "0.0.0.0:22"]
    assert sshd_mode.get("ciphers") == ["aes256-gcm@openssh.com,chacha20-poly1305@openssh.com,aes256-ctr,aes128-gcm@openssh.com,aes128-ctr"]
    assert ('addressfamily' in sshd_mode) is True
    assert ('test' in sshd_mode) is False


SSHD_CONFIGURATION_D = """
Include /etc/crypto-policies/back-ends/opensshserver.config

SyslogFacility AUTHPRIV

ChallengeResponseAuthentication no

GSSAPIAuthentication yes
GSSAPICleanupCredentials no
""".strip()


def test_sshd_configuration_d():
    sshd_config_d = SshDConfigD(context_wrap(SSHD_CONFIGURATION_D))
    assert sshd_config_d['Include'] == ['/etc/crypto-policies/back-ends/opensshserver.config']


def test_sshd_test_mode_docs():
    env = {
        'sshd_test_mode': SshdTestMode(context_wrap(SSHD_TEST_MODE)),
        'sshd_config': SshDConfig(context_wrap(SSHD_CONFIG_INPUT)),
        'sshd_config_d': SshDConfigD(context_wrap(SSHD_CONFIGURATION_D))
    }
    failed, total = doctest.testmod(ssh, globs=env)
    assert failed == 0



from insights.core.exceptions import SkipComponent
from insights.core.plugins import make_none, rule
from insights.parsers.redhat_release import RedhatRelease


@rule(RedhatRelease)
def report_none(u):
    return


@rule(RedhatRelease)
def report_make_none(u):
    return make_none()


@rule(RedhatRelease)
def report_skip_exception(u):
    raise SkipComponent()


if __name__ == "__main__":
    from insights import run

    broker = run([report_none, report_make_none], print_summary=True)



"""
Custom datasources for the number of "tasks" files under /sys/fs/cgroup/memory
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by sys_fs_cgroup_memory_tasks_number datasources """

    sys_fs_cgroup_memory_tasks_raw = simple_command("/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'")
    """ Returns the output of command ``/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'`` """


@datasource(LocalSpecs.sys_fs_cgroup_memory_tasks_raw, HostContext)
def sys_fs_cgroup_memory_tasks_number_data_datasource(broker):
    """
    This datasource provides the numeber of "tasks" file collected
    from ``/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'``.

    Typical content of ``/usr/bin/find /sys/fs/cgroup/memory -name 'tasks'`` command is::

        /sys/fs/cgroup/memory/user.slice/tasks
        /sys/fs/cgroup/memory/system.slice/rh-nginx120-nginx.service/tasks
        /sys/fs/cgroup/memory/system.slice/named.service/tasks
        /sys/fs/cgroup/memory/system.slice/rhel-push-plugin.service/tasks

    Returns:
        string: the number of "tasks" file under /sys/fs/cgroup/memory

    Raises:
        SkipComponent: When any exception occurs.
    """
    exceptions = ['no such file or directory', ]
    content = broker[LocalSpecs.sys_fs_cgroup_memory_tasks_raw].content
    if len(content) == 0 or not any(ex in content[0].lower() for ex in exceptions):
        return DatasourceProvider(content=str(len(content)), relative_path='insights_commands/sys_fs_cgroup_memory_tasks_number')
    raise SkipComponent



from insights.tests import context_wrap
from insights.parsers.sysconfig import HttpdSysconfig

HTTPD = """
# Configuration file for the httpd service.

#
# The default processing model (MPM) is the process-based
# 'prefork' model.  A thread-based model, 'worker', is also
# available, but does not work with some modules (such as PHP).
# The service must be stopped before changing this variable.
#
HTTPD=/usr/sbin/httpd.worker

#
# To pass additional options (for instance, -D definitions) to the
# httpd binary at startup, set OPTIONS here.
#
#OPTIONS=
OPTIONS1=
#
# By default, the httpd process is started in the C locale; to
# change the locale in which the server runs, the HTTPD_LANG
# variable can be set.
#
HTTPD_LANG=C
""".strip()


def test_httpd_service_conf():
    result = HttpdSysconfig(context_wrap(HTTPD))
    assert result["HTTPD"] == '/usr/sbin/httpd.worker'
    assert result.get("OPTIONS") is None
    assert result.get("OPTIONS1") == ''
    assert result['HTTPD_LANG'] == "C"



from insights.parsers.ceph_osd_tree_text import CephOsdTreeText
from insights.parsers.ceph_insights import CephInsights
from insights.parsers.ceph_cmd_json_parsing import CephOsdTree as CephOsdTreeParser
from insights.combiners import ceph_osd_tree
from insights.combiners.ceph_osd_tree import CephOsdTree
from insights.tests import context_wrap
import doctest

CEPH_INSIGHTS = """
{
  "osd_tree": {
    "nodes": [
      {
        "children": [
          -3
        ],
        "type_id": 10,
        "type": "root",
        "id": -1,
        "name": "default"
      },
      {
        "name": "daq",
        "type_id": 1,
        "id": -3,
        "pool_weights": {},
        "type": "host",
        "children": [
          2,
          1,
          0
        ]
      },
      {
        "status": "up",
        "name": "osd.0",
        "exists": 1,
        "type_id": 0,
        "reweight": 1.0,
        "crush_weight": 0.009796142578125,
        "pool_weights": {},
        "primary_affinity": 1.0,
        "depth": 2,
        "device_class": "ssd",
        "type": "osd",
        "id": 0
      },
      {
        "status": "up",
        "name": "osd.1",
        "exists": 1,
        "type_id": 0,
        "reweight": 1.0,
        "crush_weight": 0.009796142578125,
        "pool_weights": {},
        "primary_affinity": 1.0,
        "depth": 2,
        "device_class": "ssd",
        "type": "osd",
        "id": 1
      },
      {
        "status": "up",
        "name": "osd.2",
        "exists": 1,
        "type_id": 0,
        "reweight": 1.0,
        "crush_weight": 0.009796142578125,
        "pool_weights": {},
        "primary_affinity": 1.0,
        "depth": 2,
        "device_class": "ssd",
        "type": "osd",
        "id": 2
      }
    ],
    "stray": []
  },
  "osd_metadata": {
    "1": {
      "bluefs_db_size": "67108864",
      "bluestore_bdev_size": "10737418240",
      "bluestore_bdev_driver": "KernelDevice"
    }
  },
  "version": {
    "release": 14,
    "major": 0,
    "full": "ceph version 14.0.0-3517-g5322f99370 (5322f99370d629f6927b9c948522a003fc5da5bb) nautilus (dev)",
    "minor": 0
  }
}
""".strip()

CEPH_OSD_TREE_TEXT = """
ID CLASS WEIGHT  TYPE NAME       STATUS REWEIGHT PRI-AFF
-1       0.08752 root default
-9       0.02917     host ceph1
 2   hdd 0.01459         osd.2       up  1.00000 1.00000
 5   hdd 0.01459         osd.5       up  1.00000 1.00000
-5       0.02917     host ceph2
 1   hdd 0.01459         osd.1       up  1.00000 1.00000
 4   hdd 0.01459         osd.4       up  1.00000 1.00000
-3       0.02917     host ceph3
 0   hdd 0.01459         osd.0       up  1.00000 1.00000
 3   hdd 0.01459         osd.3       up  1.00000 1.00000
-7             0     host ceph_1
""".strip()

CEPH_OSD_TREE = """
{
    "nodes": [
        {
            "id": -1,
            "name": "default",
            "type": "root",
            "type_id": 10,
            "children": [
                -7,
                -3,
                -5,
                -9
            ]
        },
        {
            "id": -9,
            "name": "ceph1",
            "type": "host",
            "type_id": 1,
            "pool_weights": {},
            "children": [
                5,
                2
            ]
        },
        {
            "id": 2,
            "device_class": "hdd",
            "name": "osd.2",
            "type": "osd",
            "type_id": 0,
            "crush_weight": 0.014587,
            "depth": 2,
            "pool_weights": {},
            "exists": 1,
            "status": "up",
            "reweight": 1.000000,
            "primary_affinity": 1.000000
        },
        {
            "id": 5,
            "device_class": "hdd",
            "name": "osd.5",
            "type": "osd",
            "type_id": 0,
            "crush_weight": 0.014587,
            "depth": 2,
            "pool_weights": {},
            "exists": 1,
            "status": "up",
            "reweight": 1.000000,
            "primary_affinity": 1.000000
        },
        {
            "id": -5,
            "name": "ceph2",
            "type": "host",
            "type_id": 1,
            "pool_weights": {},
            "children": [
                4,
                1
            ]
        },
        {
            "id": 1,
            "device_class": "hdd",
            "name": "osd.1",
            "type": "osd",
            "type_id": 0,
            "crush_weight": 0.014587,
            "depth": 2,
            "pool_weights": {},
            "exists": 1,
            "status": "up",
            "reweight": 1.000000,
            "primary_affinity": 1.000000
        },
        {
            "id": 4,
            "device_class": "hdd",
            "name": "osd.4",
            "type": "osd",
            "type_id": 0,
            "crush_weight": 0.014587,
            "depth": 2,
            "pool_weights": {},
            "exists": 1,
            "status": "up",
            "reweight": 1.000000,
            "primary_affinity": 1.000000
        },
        {
            "id": -3,
            "name": "ceph3",
            "type": "host",
            "type_id": 1,
            "pool_weights": {},
            "children": [
                3,
                0
            ]
        },
        {
            "id": 0,
            "device_class": "hdd",
            "name": "osd.0",
            "type": "osd",
            "type_id": 0,
            "crush_weight": 0.014587,
            "depth": 2,
            "pool_weights": {},
            "exists": 1,
            "status": "up",
            "reweight": 1.000000,
            "primary_affinity": 1.000000
        },
        {
            "id": 3,
            "device_class": "hdd",
            "name": "osd.3",
            "type": "osd",
            "type_id": 0,
            "crush_weight": 0.014587,
            "depth": 2,
            "pool_weights": {},
            "exists": 1,
            "status": "up",
            "reweight": 1.000000,
            "primary_affinity": 1.000000
        },
        {
            "id": -7,
            "name": "ceph_1",
            "type": "host",
            "type_id": 1,
            "pool_weights": {},
            "children": []
        }
    ],
    "stray": []
}
""".strip()


def test_ceph_osd_tree_parser():
    cot = CephOsdTreeParser(context_wrap(CEPH_OSD_TREE))
    ci = CephInsights(context_wrap(CEPH_INSIGHTS))
    ret = CephOsdTree(cot, ci, None)
    assert ret["nodes"][0] == {'id': -1, 'name': 'default', 'type': 'root', 'type_id': 10, 'children': [-7, -3, -5, -9]}


def test_ceph_osd_tree_parser_2():
    cot = CephOsdTreeParser(context_wrap(CEPH_OSD_TREE))
    ci = CephInsights(context_wrap(CEPH_INSIGHTS))
    cott = CephOsdTreeText(context_wrap(CEPH_OSD_TREE_TEXT))
    ret = CephOsdTree(cot, ci, cott)
    assert ret["nodes"][0] == {'id': -1, 'name': 'default', 'type': 'root', 'type_id': 10, 'children': [-7, -3, -5, -9]}


def test_ceph_insights():
    ci = CephInsights(context_wrap(CEPH_INSIGHTS))
    cott = CephOsdTreeText(context_wrap(CEPH_OSD_TREE_TEXT))
    ret = CephOsdTree(None, ci, cott)
    assert ret["nodes"][0] == {'children': [-3], 'type_id': 10, 'type': 'root', 'id': -1, 'name': 'default'}


def test_ceph_osd_tree_text():
    cott = CephOsdTreeText(context_wrap(CEPH_OSD_TREE_TEXT))
    ret = CephOsdTree(None, None, cott)
    assert ret["nodes"][0] == {'id': '-1', 'device_class': '', 'crush_weight': '0.08752', 'name': 'default',
                               'status': '', 'reweight': '', 'primary_affinity': '', 'type': 'root',
                               'children': [-7, -3, -5, -9]}


def test_ceph_osd_tree_doc_examples():
    env = {
        'cot': CephOsdTree(None, None, CephOsdTreeText(
            context_wrap(CEPH_OSD_TREE_TEXT)))

    }
    failed, total = doctest.testmod(ceph_osd_tree, globs=env)
    assert failed == 0



"""
Krb5Configuration - files ``/etc/krb5.conf`` and ``/etc/krb5.conf.d/*``
=======================================================================

krb5 Configuration are ``/etc/krb5.conf`` and ``/etc/krb5.conf.d/*``,
and the content format is similar to ``INI config``, but they include
values that span multiple lines. Multi-line values start with a '{'
and end with a '}', and we join them together by setting the `is_squ`
variable to True while in a multi-line value.

Example:
    >>> krb5_content = '''
    [realms]
     dns_lookup_realm = false
     ticket_lifetime = 24h
     default_ccache_name = KEYRING:persistent:%{uid}
     EXAMPLE.COM = {
      kdc = kerberos.example.com
      admin_server = kerberos.example.com
     }
     pam = {
      debug = false
      krb4_convert = false
      ticket_lifetime = 36000
     }
     [libdefaults]
      dns_lookup_realm = false
      ticket_lifetime = 24h
      EXAMPLE.COM = {
       kdc = kerberos2.example.com
       admin_server = kerberos2.example.com
     }
    # renew_lifetime = 7d
    # forwardable = true
    # rdns = false
    '''.strip()

    >>> from insights.tests import context_wrap
    >>> shared = {Krb5Configuration: Krb5Configuration(context_wrap(krb5_content))}
    >>> krb5_info = shared[Krb5Configuration]
    >>> krb5_info["libdefaults"]["dnsdsd"]
    "false"
    >>> krb5_info["realms"]["EXAMPLE.COM"]["kdc"]
    "kerberos.example.com"
    >>> krb5_info.sections()
    ["libdefaults","realms"]
    >>> krb5_info.has_section("realms")
    True
    >>> krb5_info.has_option("realms", "nosuchoption")
    False
    >>> krb5_info.options("libdefaults")
    ["dns_lookup_realm","ticket_lifetime","EXAMPLE.COM"]
"""


from .. import parser, Parser, get_active_lines, LegacyItemAccess
from insights.specs import Specs


PREFIX_FOR_LIST = ("includedir", "include", "module")


def _handle_key_value(t_dict, key, value):
    """
    Function to handle key has multi value, and return the values as list.
    """
    if key in t_dict:
        val = t_dict[key]
        if isinstance(val, str):
            val = [val]
        val.append(value)
        return val
    return value


def _handle_krb5_bool(value):
    """
    Convert krb5.conf boolean
    """
    # see lib/krb5/krb/libdef_parse.c _krb5_conf_boolean()
    if value in set(["y", "yes", "true", "t", "1", "on"]):
        return True
    elif value in set(["n", "no", "false", "nil", "0", "off"]):
        return False
    else:
        # _krb5_conf_boolean() treats any other value as "False". Return
        # "None" so caller can identify this case.
        return None


@parser(Specs.krb5)
class Krb5Configuration(Parser, LegacyItemAccess):
    """
    Class for ``krb5.conf`` and ``krb5.conf.d`` configuration files.

    The Kerberos .ini format is like an ordinary .ini file except that values
    can include a multiple line key-value pair 'relation' that starts with a
    '{' and end with a '}' on a trailing line.  So we track whether we're in
    curly braces by setting `is_squ` when we enter a relation, and clearing
    it when we leave.  Please fill in the remainder of the logic here.

    Attributes:
        includedir (list): The directory list that `krb5.conf` includes via
            `includedir` directive
        include (list): The configuration file list that `krb5.conf` includes
            via `include` directive
        module (list): The module list that `krb5.conf` specifed via `module`
            directive
    """

    def parse_content(self, content):
        dict_all = {}
        is_squ = False
        section_name = ""
        squ_value = {}
        squ_section_name = ""
        section_value = {}
        self.includedir = []
        self.include = []
        self.module = []
        unchangeable_tags = []
        for line in get_active_lines(content):
            line = line.strip()
            if line.startswith(PREFIX_FOR_LIST):
                key, value = [i.strip() for i in line.split(None, 1)]
                getattr(self, key).append(value) if key in PREFIX_FOR_LIST else None
                continue
            if is_squ:
                # If in {} sub_section, get the key_value pair
                if "=" in line:
                    key, value = [i.strip() for i in line.split('=', 1)]
                    if key not in unchangeable_tags:
                        value = value.split()[0] if value else value
                        squ_value[key] = _handle_key_value(squ_value, key, value)
                    if line.endswith("*"):
                        unchangeable_tags.append(key)
                # The {} sub_section should end with },
                # if it is, set the whole value to the sub_section name,
                # and clean the flag
                else:
                    section_value[squ_section_name] = squ_value
                    is_squ = False
                    squ_section_name = ""
                    squ_value = {}
            else:
                # [XXX] means a section, get the section name and the value
                # format is dict.
                if line.startswith("[") and line.endswith("]"):
                    # If first section, just get the section name,
                    # if not, set the value to the former section and
                    # get the section name
                    section_name = line.strip("[]")
                    section_value = {}
                    if section_name:
                        dict_all[section_name] = section_value
                # key value format is XXX = YYY, store as dict
                elif "=" in line and not line.endswith("{"):
                    key, value = [i.strip() for i in line.split('=', 1)]
                    if key not in unchangeable_tags:
                        value = value.split()[0] if value else value
                        section_value[key] = _handle_key_value(section_value, key, value)
                    if line.endswith("*"):
                        unchangeable_tags.append(key)
                # The {} sub_section should start with format XXX = {
                else:
                    is_squ = True
                    squ_section_name = line.split("=")[0].strip()
        self.data = dict_all

    def sections(self):
        """
        Return a list of section names.
        """
        return self.data.keys()

    def has_section(self, section):
        """
        Indicate whether the named section is present in the configuration.
        Return True if the given section is present, and False if not present.
        """
        return section in self.data

    def options(self, section):
        """
        Return a list of option names for the given section name.
        """
        return self.data[section].keys() if self.has_section(section) else []

    def has_option(self, section, option):
        """
        Check for the existence of a given option in a given section.
        Return True if the given option is present, and False if not present.
        """
        if section not in self.data:
            return False
        return option in self.data[section]

    def getboolean(self, section, option):
        """Parse option as bool

        Returns None is not a krb5.conf boolean string.
        """
        value = self.data[section][option]
        return _handle_krb5_bool(value)



#!/usr/bin/env python

import json
import logging
import os
import sys

from insights.parsr import query

log = logging.getLogger(__name__)
logging.basicConfig(format='%(message)s', level=logging.INFO)

coverage_json = "coverage.json"

if len(sys.argv) < 2:
    log.error("Provide changed files.")
    sys.exit(1)

if not os.path.exists(coverage_json):
    log.error("'coverage.json' does not exist.")
    sys.exit(1)

coverage_report = None
with open(coverage_json) as fp:
    coverage_report = query.from_dict(json.load(fp))

if not coverage_report:
    log.error("No coverage report.")
    sys.exit(1)

changed_files = []
with open(sys.argv[1]) as fp:
    changed_files = list(filter(None, (l.strip() for l in fp.read().splitlines())))

if not changed_files:
    log.info("===================================================================")
    log.info("No coverage check is required for this change.")
    sys.exit(0)

log.info(f"File(s) need to check coverage:")
for chf in changed_files:
    log.info(f"  - {chf}")

log.info("===================================================================")
log.info(f"Total Coverage: {coverage_report.totals.percent_covered_display.value}%")
log.info("-------------------------------------------------------------------")

okay_flag = True
missed_cov_files = []
log.info("File(s) Coverage:")
for chf in changed_files:
    cov = coverage_report.find(chf).summary.percent_covered_display.value
    if cov:
        if int(cov) != 100:
            okay_flag = False
            missed_cov_files.append((chf, cov))
        else:
            log.info(f"  - {chf}: {cov}%")


if not okay_flag:
    log.info("")
    for chf, cov in missed_cov_files:
        log.info(f"  - {chf}: {cov}%")
    log.info("-------------------------------------------------------------------")
    log.info(f"Check details of coverage less than 100% with 'pytest --cov'.")
    sys.exit(1)

log.info("-------------------------------------------------------------------")
log.info("Coverage of this change is fine.")
sys.exit(0)



import base64
import copy
import hashlib
import logging
import os
import pkgutil
import sys
import tempfile

import six

import insights.client.apps.ansible
from insights.client.apps.ansible.playbook_verifier.serializer import PlaybookSerializer
from insights.client.apps.ansible.playbook_verifier.contrib import gnupg
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel import yaml
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel.yaml.comments import CommentedMap, CommentedSeq
from insights.client.apps.ansible.playbook_verifier.contrib.ruamel_yaml.ruamel.yaml.scalarint import ScalarInt
from insights.client.constants import InsightsConstants as constants


__all__ = ("load_playbook_yaml", "verify", "PlaybookVerificationError")

yaml = yaml.YAML(typ='rt')
yaml.indent(mapping=2, sequence=4, offset=2)
yaml.default_flow_style = False
yaml.preserve_quotes = True
yaml.width = 200

PLAYBOOK_SIGNATURE_LABEL = 'insights_signature'
PLAYBOOK_DYNAMIC_LABELS = ['hosts', 'vars']

PUBLIC_KEY_PATH = pkgutil.get_data(insights.client.apps.ansible.__name__, 'playbook_verifier/public.gpg')

logger = logging.getLogger(__name__)
logging.getLogger('insights.client.apps.ansible.playbook_verifier.contrib').setLevel(logging.INFO)


class PlaybookVerificationError(Exception):
    """Exception raised when playbook verification fails."""
    def __init__(self, message="Could not verify Ansible playbook"):
        self.message = message
        super(PlaybookVerificationError, self).__init__(self.message)

    def __str__(self):
        return self.message


def load_playbook_yaml(playbook):
    """Parse a string into a dictionary.

    :param playbook: The Ansible playbook
    :type playbook: str

    :returns: The playbook dictionary
    :rtype: list[dict]

    :raises PlaybookVerificationError: The input is empty or corrupted.
    """
    if not playbook:
        raise PlaybookVerificationError("Cannot load a playbook from an empty input.")

    try:
        return yaml.load(playbook)
    except Exception:
        raise PlaybookVerificationError("Could not load the playbook.")


def normalize_play_py2(play):
    """In Python 2, get rid of any default unicode values.

    :param play: The Ansible play.
    :type play: dict | yaml.comments.CommentedMap

    :returns: Play with Unicode characters removed.
    :rtype: CommentedMap
    """
    result = CommentedMap()
    for key, value in play.iteritems():
        if isinstance(value, CommentedMap):
            result[key] = CommentedMap(normalize_play_py2(value))
            continue

        if isinstance(value, CommentedSeq):
            new_sequence = CommentedSeq()
            for item in value:
                if isinstance(item, six.text_type):
                    new_sequence.append(item.encode('ascii', 'ignore'))
                    continue
                if isinstance(item, CommentedMap):
                    new_sequence.append(normalize_play_py2(item))
                    continue
                new_sequence.append(item)

            result[key] = new_sequence
            continue

        if isinstance(value, six.text_type):
            result[key] = value.encode('ascii', 'ignore')
            continue

        if isinstance(value, ScalarInt):
            result[key] = int(value)
            continue

        result[key] = value

    return result


def serialize_play(play):
    """Convert the play object to bytes.

    :param play: The Ansible play.
    :type play: dict
    :returns: Serialized play.
    :rtype: bytes
    """
    if six.PY2:
        return str(normalize_play_py2(play)).encode("utf-8")
    if sys.version_info < (3, 12):
        return str(play).encode("utf-8")
    return PlaybookSerializer.serialize(play).encode("utf-8")


def hash_play(serialized_play):
    """Create SHA256 hash of a play.

    :param serialized_play: The Ansible play.
    :type serialized_play: bytes
    :returns: Hashed play.
    :rtype: bytes
    """
    sha = hashlib.sha256()
    sha.update(serialized_play)
    return sha.digest()


def get_public_key(gpg):
    """Import public GPG key.

    :param gpg: Path to the GPG key.
    :type gpg: gnupg.GPG
    :returns: Import result
    :rtype: ...
    """
    if not PUBLIC_KEY_PATH:
        raise PlaybookVerificationError(message="Public key file not found.")

    public_key = PUBLIC_KEY_PATH
    import_results = gpg.import_keys(public_key)
    if import_results.count < 1:
        raise PlaybookVerificationError(message="Public key import failed.")

    return import_results


def exclude_dynamic_elements(play):
    """Remove dynamic elements from the Ansible play.

    Some parts of the Ansible play are dynamic (e.g. host systems) and cannot be signed,
    because the signature would have to be unique for every play that is generated.

    This function removes these variable sections.

    :returns: New play dictionary without the unwanted values.
    :rtype: dict

    :raises PlaybookVerificationError:
    """
    if 'insights_signature_exclude' not in play.get('vars', {}):
        raise PlaybookVerificationError(
            "Play does not have the key 'vars/insights_signature_exclude', "
            "dynamic exclusion cannot be performed."
        )

    exclusions = play['vars']['insights_signature_exclude'].split(',')  # type: list[str]
    result = copy.deepcopy(play)  # type: dict[str, ...]

    for element in exclusions:
        elements = [string for string in element.split("/") if string != '']  # type: list[str]

        if len(elements) == 1 and elements[0] in PLAYBOOK_DYNAMIC_LABELS:
            try:
                del result[elements[0]]
            except Exception:
                raise PlaybookVerificationError(
                    "Dynamic key '{key}' does not exist and could not be excluded.".format(key=element)
                )
            continue

        if len(elements) == 2 and elements[0] in PLAYBOOK_DYNAMIC_LABELS:
            try:
                del result[elements[0]][elements[1]]
            except Exception:
                raise PlaybookVerificationError(
                    "Dynamic key '{key}' does not exist and could not be excluded.".format(key=element)
                )
            continue

        raise PlaybookVerificationError(
            "Dynamic key '{key}' cannot be excluded from verification.".format(key=element)
        )

    return result


def execute_verification(play, encoded_signature):
    """Use GPG to verify the play.

    :param play: The Ansible play.
    :type play: dict
    :param encoded_signature: The base64-encoded signature.
    :type encoded_signature: str

    :returns: Result of the GPG verification and a hash of the play.
    :rtype: Tuple[..., bytes]
    """
    play_name = play.get("name", "unnamed")  # type: str
    logger.debug("Play '{play_name}' is being validated".format(play_name=play_name))

    gpg = gnupg.GPG(gnupghome=constants.insights_core_lib_dir)
    serialized_play = serialize_play(play)
    play_hash = hash_play(serialized_play)
    decoded_signature = base64.b64decode(encoded_signature)

    # load public key
    get_public_key(gpg)

    fd, fn = tempfile.mkstemp()
    os.write(fd, decoded_signature)
    os.close(fd)

    result = gpg.verify_data(fn, play_hash)
    os.unlink(fn)

    logger.debug("Play '{play_name}' validation result: valid={valid}, status={status}".format(
        play_name=play_name, valid=result.valid, status=result.status
    ))
    return result, play_hash


def verify_play(play):
    """Verify the signature in a play.

    :param play: The Ansible play.
    :type play: dict

    :returns: Result of the GPG verification and a hash of the play.
    :rtype: Tuple[..., bytes]
    """
    if not isinstance(play.get("vars", None), dict):
        raise PlaybookVerificationError("Play doesn't have a section 'vars'.")
    if play.get("vars", {}).get(PLAYBOOK_SIGNATURE_LABEL, None) is None:
        raise PlaybookVerificationError("Play doesn't contain the Insights signature.")

    cleaned_play = exclude_dynamic_elements(play)  # type: dict
    encoded_signature = play["vars"][PLAYBOOK_SIGNATURE_LABEL]  # type: str

    return execute_verification(cleaned_play, encoded_signature)


def get_play_revocation_list(revoked_plays_yaml):
    """
    Load the list of revoked play hashes from the egg.

    :param revoked_plays_yaml: The YAML containing hashes of revoked plays.
    :type revoked_plays_yaml: bytes
    :returns: Revocation entries in a form of `name:hash`.
    :rtype: list[dict[str, str]]
    """
    try:
        # We know the structure of the YAML will stay like this:
        # > - name: revocation list
        # >   timestamp: ...
        # >   vars:
        # >     insights_signature_exclude: ...
        # >     insights_signature: !!binary | ...
        # >   revoked_playbooks:
        # >     - name: ...
        # >       hash: ...
        # >     - name: ...
        # >       hash: ...
        # There will never be more than one top-level object, we can safely do [0].
        revoked_plays = yaml.load(revoked_plays_yaml)[0]  # type: dict
    except Exception:
        raise PlaybookVerificationError("Could not load play revocation list.")

    verified, _ = verify_play(revoked_plays)

    if not verified:
        raise PlaybookVerificationError("List of revocation signatures is invalid.")

    revocation_list = revoked_plays.get("revoked_playbooks", [])  # type: list[str]
    return revocation_list


def verify(play):
    """Verify the GPG-signed Ansible play.

    :param play: Unverified Ansible play.
    :type play: dict
    :returns: Verified Ansible play.
    :rtype: dict
    :raises PlaybookVerificationError: An error occurred when trying to verify the play.
    """
    play_name = play.get("name", "unnamed")  # type: str
    logger.info("Play '{name}' is being verified.".format(name=play_name))

    if not play:
        raise PlaybookVerificationError("Empty plays cannot be verified.")

    revocation_list_file_content = pkgutil.get_data('insights', 'revoked_playbooks.yaml')  # type: bytes
    revocation_list = get_play_revocation_list(revocation_list_file_content)  # type: list[dict[str, str]]
    logger.debug("List of revoked playbooks was loaded.")

    verified, play_hash = verify_play(play)  # type: ..., str

    if not verified:
        raise PlaybookVerificationError(message="Play '{0}' has invalid signature".format(play_name))

    for revoked_item in revocation_list:
        if play_hash == bytearray.fromhex(revoked_item['hash']):
            raise PlaybookVerificationError(message="Play '{0}' is on the revoked list.".format(play_name))

    logger.info("Play '{name}' passed verification.".format(name=play_name))
    return play



import doctest
from insights.parsers import ls_usr_lib64
from insights.parsers.ls_usr_lib64 import LsUsrLib64
from insights.tests import context_wrap

LS_USR_LIB64 = """
total 447460
dr-xr-xr-x. 150 0 0    77824 Jul 30 16:39 .
drwxr-xr-x.  13 0 0     4096 Apr 30  2017 ..
drwxr-xr-x.   3 0 0       20 Nov  3  2016 krb5
-rwxr-xr-x.   1 0 0   155464 Oct 28  2016 ld-2.17.so
drwxr-xr-x.   3 0 0       20 Jun 10  2016 ldb
lrwxrwxrwx.   1 0 0       10 Apr 30  2017 ld-linux-x86-64.so.2 -> ld-2.17.so
lrwxrwxrwx.   1 0 0       21 Apr 30  2017 libabrt_dbus.so.0 -> libabrt_dbus.so.0.0.1
"""


def test_ls_usr_lib64():
    ls_usr_lib64 = LsUsrLib64(context_wrap(LS_USR_LIB64, path='insights_commands/ls_-lan_.usr.lib64'))
    assert len(ls_usr_lib64.dirs_of('/usr/lib64')) == 4
    ret = ls_usr_lib64.dir_entry('/usr/lib64', 'libabrt_dbus.so.0')
    assert ret == {
            'type': 'l',
            'perms': 'rwxrwxrwx.',
            'links': 1,
            'owner': '0',
            'group': '0',
            'size': 21,
            'date': 'Apr 30  2017',
            'name': 'libabrt_dbus.so.0',
            'link': 'libabrt_dbus.so.0.0.1',
            'raw_entry': 'lrwxrwxrwx.   1 0 0       21 Apr 30  2017 libabrt_dbus.so.0 -> libabrt_dbus.so.0.0.1',
            'dir': '/usr/lib64'
    }


def test_ls_usr_lib64_doc_examples():
    env = {
        'ls_usr_lib64': LsUsrLib64(context_wrap(LS_USR_LIB64, path='insights_commands/ls_-lan_.usr.lib64')),
    }
    failed, total = doctest.testmod(ls_usr_lib64, globs=env)
    assert failed == 0



"""
NVIDIA Related Information
==========================

NvidiaSmiL - command ``/usr/bin/nvidia-smi -L``
------------------------------------------------
"""

from insights.core import Parser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.nvidia_smi_l)
class NvidiaSmiL(Parser, list):
    """
    Prase for output of command `/usr/bin/nvidia-smi -L`. This command lists
    each of the NVIDIA GPUs in the system, along with their UUIDs.

    The GPU info shown in each line will be parsed as follows::

        model (string):   The gpu model
        uuid (string):    The gpu uuid

    Raises:
        ParseException: When run into unparsable gpu line
        SkipComponent: When content is empty or no parsable content

    Sample Content::

        GPU 0: NVIDIA A100-PCIE-40GB (UUID: GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838)
        GPU 1: NVIDIA A100-PCIE-40GB (UUID: GPU-c9bd25dc-c0c4-3ab6-8f7f-3ad16d6bde4a)

    Examples::
        >>> gpus.gpu_count
        2
        >>> "NVIDIA A100-PCIE-40GB" in gpus.gpu_models
        True
        >>> gpus[0]
        {'model': 'NVIDIA A100-PCIE-40GB', 'uuid': 'GPU-63110aaa-3561-c8f5-e125-4ab40bbcf838'}
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty content")

        _gpu_index_validator = 0

        for line in content:
            if line.startswith("GPU ") and "(UUID: " in line and line.endswith(')'):
                spl_line = line.split("(UUID: ")
                spl_left = spl_line[0].split(': ')
                if not len(spl_left) == 2:
                    raise ParseException("Unparsable GPU model: %s" % line)

                gpu_id = spl_left[0].split()[-1]
                if not (gpu_id.isdigit() and int(gpu_id) == _gpu_index_validator):
                    raise ParseException("Unparsable GPU id: %s" % line)
                _gpu_index_validator += 1

                gpu_model = spl_left[-1].strip()
                gpu_uuid = spl_line[-1].strip(') ')
                if not (gpu_model and gpu_uuid):
                    raise ParseException("Unparsable GPU line: %s" % line)

                self.append({
                    "model": gpu_model,
                    "uuid": gpu_uuid,
                })

        if len(self) < 1:
            raise ParseException("Empty GPU info after parse: %s" % content)

    @property
    def gpu_count(self):
        """
        str: Returns the GPU count
        """
        return len(self)

    @property
    def gpu_models(self):
        """
        str: Returns the GPUs model set
        """
        return set([gpu["model"] for gpu in self])



import doctest

from insights.parsers import ls_sys_firmware
from insights.parsers.ls_sys_firmware import LsSysFirmware
from insights.tests import context_wrap


LS_SYS_FIRMWARE = """
/sys/firmware:
total 0
drwxr-xr-x.  5 0 0 0 Dec 22 17:56 .
dr-xr-xr-x. 13 0 0 0 Dec 22 17:56 ..
drwxr-xr-x.  5 0 0 0 Dec 22 17:56 acpi
drwxr-xr-x.  3 0 0 0 Dec 22 17:57 dmi
drwxr-xr-x. 10 0 0 0 Dec 22 17:57 memmap

/sys/firmware/acpi:
total 0
drwxr-xr-x. 5 0 0    0 Dec 22 17:56 .
drwxr-xr-x. 5 0 0    0 Dec 22 17:56 ..
drwxr-xr-x. 6 0 0    0 Feb 10 15:54 hotplug
drwxr-xr-x. 2 0 0    0 Feb 10 15:54 interrupts
-r--r--r--. 1 0 0 4096 Feb 10 15:54 pm_profile
drwxr-xr-x. 3 0 0    0 Dec 22 17:56 tables
"""

LS_ALZR_SYS_FIRMWARE = """
/sys/firmware:
total 0
drwxr-xr-x.  7 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 .
dr-xr-xr-x. 12 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 ..
drwxr-xr-x.  5 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 acpi
drwxr-xr-x.  2 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 devicetree
drwxr-xr-x.  4 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 dmi
drwxr-xr-x.  3 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 efi
-r--------.  1 root root system_u:object_r:sysfs_t:s0 657 Oct  1 10:17 fdt

/sys/firmware/acpi:
total 0
drwxr-xr-x. 5 root root system_u:object_r:sysfs_t:s0    0 Sep 30 16:58 .
drwxr-xr-x. 7 root root system_u:object_r:sysfs_t:s0    0 Sep 30 16:58 ..
drwxr-xr-x. 2 root root system_u:object_r:sysfs_t:s0    0 Oct  1 10:17 bgrt
drwxr-xr-x. 5 root root system_u:object_r:sysfs_t:s0    0 Oct  1 10:17 hotplug
-r--r--r--. 1 root root system_u:object_r:sysfs_t:s0 4096 Oct  1 10:17 pm_profile
drwxr-xr-x. 4 root root system_u:object_r:sysfs_t:s0    0 Sep 30 16:58 tables
"""


def test_ls_sys_firmware():
    ls = LsSysFirmware(context_wrap(LS_SYS_FIRMWARE))
    assert "acpi" not in ls
    assert "/sys/firmware/acpi" in ls
    assert ls.dirs_of("/sys/firmware") == [".", "..", "acpi", "dmi", "memmap"]
    assert ls.files_of("/sys/firmware/acpi") == ["pm_profile"]

    ls_alzr_sys_firmware = LsSysFirmware(context_wrap(LS_ALZR_SYS_FIRMWARE))
    assert "/sys/firmware/acpi" in ls_alzr_sys_firmware
    assert "/sys/firmware/dmi" not in ls_alzr_sys_firmware
    assert ls_alzr_sys_firmware.get("/sys/firmware")["entries"].get("efi") == {
        "type": "d",
        "perms": "rwxr-xr-x.",
        "links": 3,
        "owner": "root",
        "group": "root",
        "size": 0,
        "se_user": "system_u",
        "se_role": "object_r",
        "se_type": "sysfs_t",
        "se_mls": "s0",
        "name": "efi",
        "date": "Sep 30 16:58",
        "raw_entry": "drwxr-xr-x.  3 root root system_u:object_r:sysfs_t:s0   0 Sep 30 16:58 efi",
        "dir": "/sys/firmware",
    }


def test_doc_examples():
    env = {"ls_sys_firmware": LsSysFirmware(context_wrap(LS_SYS_FIRMWARE))}
    failed, total = doctest.testmod(ls_sys_firmware, globs=env)
    assert failed == 0



import doctest

import insights.combiners.rhsm_release as rhsm_release_module
from insights.combiners.rhsm_release import RhsmRelease
from insights.parsers.rhsm_releasever import RhsmReleaseVer
from insights.parsers.rhui_release import RHUIReleaseVer
from insights.tests import context_wrap

RHSM_RELEASE = """
{"releaseVer": "7.6"}
""".strip()

RHSM_RELEASE_EMPTY = """
{"releaseVer": null}
""".strip()

RHUI_RELEASE = """8.6"""

RHUI_EMPTY_RELEASE = ""

RHSM_PARSER = RhsmReleaseVer(context_wrap(RHSM_RELEASE))
RHSM_EMPTY_PARSER = RhsmReleaseVer(context_wrap(RHSM_RELEASE_EMPTY))
RHUI_PARSER = RHUIReleaseVer(context_wrap(RHUI_RELEASE))
RHUI_EMPTY_PARSER = RHUIReleaseVer(context_wrap(RHUI_EMPTY_RELEASE))


def test_with_rhsm():
    rhsm_release = RhsmRelease(RHSM_PARSER, None)
    assert rhsm_release.set == '7.6'
    assert rhsm_release.major == 7
    assert rhsm_release.minor == 6


def test_with_rhui_release():
    rhsm_release = RhsmRelease(None, RHUI_PARSER)
    assert rhsm_release.set == '8.6'
    assert rhsm_release.major == 8
    assert rhsm_release.minor == 6


def test_with_all():
    rhsm_release = RhsmRelease(RHSM_PARSER, RHUI_PARSER)
    assert rhsm_release.set == '7.6'
    assert rhsm_release.major == 7
    assert rhsm_release.minor == 6


def test_with_all_but_rhsm_is_empty():
    rhsm_release = RhsmRelease(RHSM_EMPTY_PARSER, RHUI_PARSER)
    assert rhsm_release.set == '8.6'
    assert rhsm_release.major == 8
    assert rhsm_release.minor == 6


def test_with_all_but_all_are_empty():
    rhsm_release = RhsmRelease(RHSM_EMPTY_PARSER, RHUI_EMPTY_PARSER)
    assert rhsm_release.set is None
    assert rhsm_release.major is None
    assert rhsm_release.minor is None

    rhsm_release = RhsmRelease(RHSM_EMPTY_PARSER, None)
    assert rhsm_release.set is None
    assert rhsm_release.major is None
    assert rhsm_release.minor is None


def test_doc_examples():
    env = {
        'rhsm_release': RhsmRelease(RHSM_PARSER, None),
    }
    failed, total = doctest.testmod(rhsm_release_module, globs=env)
    assert failed == 0



"""
Parsers for ``modinfo <module_name>``
=====================================

KernelModulesInfo - Command ``modinfo filtered_modules``
--------------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


class ModInfo(dict):
    """
    Base class for the information about a kernel module, the module
    info will be stored in dictionary format. Besides of that, the following
    utility properties are provided as well.

    """
    @classmethod
    def from_content(cls, content):
        """
        A classmethod to generated a `ModInfo` object from the given `content`
        list. Two more keys `module_name` and `module_deps` will be created as
        well per the `content`.

        Raises:
            SkipComponent: When nothing need to check to a dict.
        """
        if not content:
            raise SkipComponent("No Contents")

        data = {}
        for line in content:
            line = line.strip()
            if ':' in line:
                key, value = [l.strip() for l in line.split(':', 1)]
                if key not in data:
                    data[key] = value
                else:
                    old_val = data[key]
                    data[key] = [old_val] if isinstance(old_val, str) else old_val
                    data[key].append(value)

        if not data:
            raise SkipComponent("No Parsed Contents")

        data['module_deps'] = list(data.get('depends', '').split(','))
        data['module_name'] = data.get('filename', '').rsplit('/')[-1].split('.')[0]

        return cls(data)

    @property
    def module_name(self):
        """
        (str): This will return kernel module name when set, else empty str.
        """
        return self.get('module_name', '')

    @property
    def module_path(self):
        """
        (str): This will return kernel module path when set, else `None`.
        """
        return self.get('filename', '')

    @property
    def module_deps(self):
        """
        (list): This will return the list of kernel modules depend on the kernel
                module when set, else `[]`.
        """
        return self.get('module_deps', [])

    @property
    def module_firmware(self):
        """
        (list): This will return the list of firmwares used by this module
                when set, else `[]`.
        """
        return self.get('firmware', [])

    @property
    def module_alias(self):
        """
        (list): This will return the list of alias to this kernel  module
                when set, else `[]`.
        """
        return self.get('alias', [])

    @property
    def module_parm(self):
        """
        (list): This will return the list of parms for this kernel module
                when set, else `[]`.
        """
        return self.get('parm', [])

    @property
    def module_version(self):
        """
        (str): This will return the kernel module version when set, else empty string.
        """
        return self.get('version', '')

    @property
    def module_signer(self):
        """
        (str): This will return the signer of kernel module when set, else empty string.
        """
        return self.get('signer', '')

    @property
    def module_details(self):
        """
        (dict): This will return the kernel module details when set.
        """
        return self


@parser(Specs.modinfo_filtered_modules)
class KernelModulesInfo(CommandParser, dict):
    """
    Class to parse the information about filtered kernel modules collected by
    "modinfo filtered_modules". The result will be stored in a dictionary.
    The key is the module name, the value is a instance of ModInfo with more
    details.

    Sample output::

        filename:       /lib/modules/3.10.0-957.10.1.el7.x86_64/kernel/drivers/net/vmxnet3/vmxnet3.ko.xz
        version:        1.4.14.0-k
        license:        GPL v2
        description:    VMware vmxnet3 virtual NIC driver
        author:         VMware, Inc.
        retpoline:      Y
        rhelversion:    7.6
        srcversion:     7E672688ACACBDD2E363B63
        alias:          pci:v000015ADd000007B0sv*sd*bc*sc*i*
        depends:
        intree:         Y
        vermagic:       3.10.0-957.10.1.el7.x86_64 SMP mod_unload modversions
        signer:         Red Hat Enterprise Linux kernel signing key
        sig_key:        A5:70:18:DF:B6:C9:D6:1F:CF:CE:0A:3D:02:8B:B3:69:BD:76:CA:ED
        sig_hashalgo:   sha256
        filename:       /lib/modules/3.10.0-993.el7.x86_64/kernel/drivers/net/ethernet/intel/i40e/i40e.ko.xz
        firmware:       i40e/i40e-e2-7.13.1.0.fw
        firmware:       i40e/i40e-e1h-7.13.1.0.fw
        version:        2.3.2-k
        license:        GPL
        description:    Intel(R) Ethernet Connection XL710 Network Driver
        author:         Intel Corporation, <e1000-devel@lists.sourceforge.net>
        retpoline:      Y
        rhelversion:    7.7
        srcversion:     DC5C250666ADD8603966656
        alias:          pci:v00008086d0000158Bsv*sd*bc*sc*i*
        alias:          pci:v00008086d0000158Asv*sd*bc*sc*i*
        depends:        ptp
        intree:         Y
        vermagic:       3.10.0-993.el7.x86_64 SMP mod_unload modversions
        signer:         Red Hat Enterprise Linux kernel signing key
        sig_key:        81:7C:CB:07:72:4E:7F:B8:15:24:10:F9:27:2D:AA:CF:80:3E:CE:59
        sig_hashalgo:   sha256
        parm:           debug:Debug level (0=none,...,16=all), Debug mask (0x8XXXXXXX) (uint)
        parm:           int_mode: Force interrupt mode other than MSI-X (1 INT#x; 2 MSI) (int)

    Raises:
        SkipComponent: When nothing need to parse.

    Examples:
        >>> from insights.core.filters import add_filter
        >>> from insights.specs import Specs
        >>> add_filter(Specs.modinfo_modules, 'i40e')
        >>> add_filter(Specs.modinfo_modules, 'vmxnet3')
        >>> type(mods_info)
        <class 'insights.parsers.modinfo.KernelModulesInfo'>
        >>> 'i40e' in mods_info
        True
        >>> mods_info['i40e'].module_version
        '2.3.2-k'
        >>> mods_info['i40e'].module_path
        '/lib/modules/3.10.0-993.el7.x86_64/kernel/drivers/net/ethernet/intel/i40e/i40e.ko.xz'
        >>> sorted(mods_info['i40e'].module_firmware)
        ['i40e/i40e-e1h-7.13.1.0.fw', 'i40e/i40e-e2-7.13.1.0.fw']
        >>> sorted(mods_info['i40e'].module_alias)
        ['pci:v00008086d0000158Asv*sd*bc*sc*i*', 'pci:v00008086d0000158Bsv*sd*bc*sc*i*']
        >>> sorted(mods_info['i40e'].module_parm)
        ['debug:Debug level (0=none,...,16=all), Debug mask (0x8XXXXXXX) (uint)', 'int_mode: Force interrupt mode other than MSI-X (1 INT#x; 2 MSI) (int)']
        >>> 'vmxnet3' in mods_info
        True

    Attributes:
        retpoline_y (set): A set of names of the modules with the attribute "retpoline: Y".
        retpoline_n (set): A set of names of the modules with the attribute "retpoline: N".
    """
    def parse_content(self, content):
        if (not content) or (not self.file_path):
            raise SkipComponent("No Contents")

        self.retpoline_y = set()
        self.retpoline_n = set()
        idx = None
        for i, line in enumerate(content):
            if line.startswith('filename:'):
                if idx is not None:
                    m = ModInfo.from_content(content[idx:i])
                    name = m.module_name
                    self[name] = m
                    self.retpoline_y.add(name) if m.get('retpoline') == 'Y' else None
                    self.retpoline_n.add(name) if m.get('retpoline') == 'N' else None
                idx = i
        if idx is not None and idx < len(content):
            m = ModInfo.from_content(content[idx:])
            name = m.module_name
            self[name] = m
            self.retpoline_y.add(name) if m.get('retpoline') == 'Y' else None
            self.retpoline_n.add(name) if m.get('retpoline') == 'N' else None

        if len(self) == 0:
            raise SkipComponent("No Parsed Contents")



"""
Get CIFS Info
=============

This module contains the following parsers:

CIFSDebugData - file ``/proc/fs/cifs/DebugData``
------------------------------------------------
"""

from insights.core import TextFileOutput
from insights.specs import Specs
from insights.core.plugins import parser


@parser(Specs.cifs_debug_data)
class CIFSDebugData(TextFileOutput):
    r"""
    Parse the ``/proc/fs/cifs/DebugData`` file.
    Filter is added to the spec.

    Sample input of full file::

        Display Internal CIFS Data Structures for Debugging
        ---------------------------------------------------
        CIFS Version 2.41
        Features: DFS,SMB_DIRECT,STATS,DEBUG,ALLOW_INSECURE_LEGACY,CIFS_POSIX,UPCALL(SPNEGO),XATTR,ACL
        CIFSMaxBufSize: 16384
        Active VFS Requests: 0

        Servers:
        1) ConnectionId: 0x4 Hostname: CVXDFAVB1
        Number of credits: 512 Dialect 0x302 signed
        TCP status: 1 Instance: 22
        Local Users To Server: 1 SecMode: 0x3 Req On Wire: 0
        In Send: 0 In MaxReq Wait: 0
        DFS origin full path: \\ad.abc.com\DFSROOT\name1
        DFS leaf full path:   \\CVXDFAVB1\DFSRoot\name1

            Sessions:
            1) Address: 10.0.96.196 Uses: 1 Capability: 0x300067    Session Status: 1
            Security type: RawNTLMSSP  SessionId: 0x2d86cec00089d
            User: 1339601402 Cred User: 0

            Shares:
            0) IPC: \\CVXDFAVB1\IPC$ Mounts: 1 DevInfo: 0x0 Attributes: 0x0
            PathComponentMax: 0 Status: 1 type: 0 Serial Number: 0x0
            Share Capabilities: None    Share Flags: 0x30
            tid: 0x5    Maximal Access: 0x11f01ff

            1) \\CVXDFAVB1\name1 Mounts: 1 DevInfo: 0x60120 Attributes: 0xc500ff
            PathComponentMax: 255 Status: 1 type: DISK Serial Number: 0x16f0426b
            Share Capabilities: None Aligned, Partition Aligned,    Share Flags: 0x0
            tid: 0x1    Optimal sector size: 0x200  Maximal Access: 0x1f01ff


            Server interfaces: 2
            1)  Speed: 10000000000 bps
                Capabilities: rss
                IPv4: 10.9.212.196

            2)  Speed: 10000000000 bps
                Capabilities: rss
                IPv4: 10.0.96.96


            MIDs:
        --

    Examples:
        >>> from insights.core.filters import add_filter
        >>> from insights.specs import Specs
        >>> add_filter(Specs.cifs_debug_data, 'abc_def_fsf')
        >>> type(cifs_dd_obj)
        <class 'insights.parsers.cifs.CIFSDebugData'>
        >>> cifs_dd_obj.last_scan("abc_def_line", "abc_def_fsf")
    """
    pass



from .. import Parser, parser, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.ovirt_engine_confd)
class OvirtEngineConfd(LegacyItemAccess, Parser):

    def parse_content(self, content):
        self.data = dict((k.strip('" '), v.strip('" '))
                         for k, _, v in [l.partition("=") for l in content])



from insights.parsers.cinder_log import CinderApiLog, CinderVolumeLog
from insights.tests import context_wrap

from datetime import datetime

api_log = """
2019-04-16 21:01:02.072 21 INFO cinder.api.openstack.wsgi [req-e7b42a20-4109-45b7-b18a-ae5ad4831909 - - - - -] OPTIONS http://controller-0.internalapi.localdomain/
2019-04-16 21:01:02.073 21 DEBUG cinder.api.openstack.wsgi [req-e7b42a20-4109-45b7-b18a-ae5ad4831909 - - - - -] Empty body provided in request get_body /usr/lib/python2.7/site-packages/cinder/api/openstack/wsgi.py:718
2019-04-16 21:01:02.073 21 DEBUG cinder.api.openstack.wsgi [req-e7b42a20-4109-45b7-b18a-ae5ad4831909 - - - - -] Calling method '<bound method VersionsController.all of <cinder.api.versions.VersionsController object at 0x7f44fe7ee1d0>>' _process_stack /usr/lib/python2.7/site-packages/cinder/api/openstack/wsgi.py:871
2019-04-16 21:01:02.074 21 INFO cinder.api.openstack.wsgi [req-e7b42a20-4109-45b7-b18a-ae5ad4831909 - - - - -] http://controller-0.internalapi.localdomain/ returned with HTTP 300
2019-04-16 21:01:04.079 22 INFO cinder.api.openstack.wsgi [req-43db684e-f4eb-4b23-9f8e-82beb95ab0b7 - - - - -] OPTIONS http://controller-0.internalapi.localdomain/
"""

CINDER_LOG = """
2015-06-19 07:31:41.020 7947 DEBUG cinder.openstack.common.periodic_task [-] Running periodic task VolumeManager._publish_service_capabilities run_periodic_tasks /usr/lib/python2.7/site-packages/cinder/openstack/common/periodic_task.py:178
2015-06-19 07:31:41.022 7947 DEBUG cinder.manager [-] Notifying Schedulers of capabilities ... _publish_service_capabilities /usr/lib/python2.7/site-packages/cinder/manager.py:128
2015-06-19 07:31:41.025 7947 DEBUG cinder.openstack.common.periodic_task [-] Running periodic task VolumeManager._report_driver_status run_periodic_tasks /usr/lib/python2.7/site-packages/cinder/openstack/common/periodic_task.py:178
2015-06-19 07:31:41.026 7947 INFO cinder.volume.manager [-] Updating volume status
2015-11-27 11:22:45.416 16656 INFO oslo.messaging._drivers.impl_rabbit [-] Connecting to AMQP server on 169.254.10.22:5672
2015-11-27 11:22:45.426 16656 INFO oslo.messaging._drivers.impl_rabbit [-] Connected to AMQP server on 169.254.10.22:5672
2015-11-27 11:23:07.146 16656 INFO cinder.volume.manager [req-a8c22cdb-e21b-497f-affe-9380478decae - - - - -] Updating volume status
2015-11-27 11:23:07.148 16656 WARNING cinder.volume.manager [req-a8c22cdb-e21b-497f-affe-9380478decae - - - - -] Unable to update stats, LVMISCSIDriver -2.0.0 (config name rbd) driver is uninitialized.
"""


def test_cinder_api_log():
    log = CinderApiLog(context_wrap(api_log))
    assert len(log.get(["req-"])) == 5
    assert len(log.get("e7b42a20-4109-45b7-b18a-ae5ad4831909")) == 4
    assert len(list(log.get_after(datetime(2019, 4, 16, 21, 0, 0)))) == 5


def test_get_cinder_log():
    log = CinderVolumeLog(context_wrap(CINDER_LOG))
    assert len(log.lines) == 8
    assert sorted([i['raw_message'] for i in log.get('cinder.volume.manager')]) == sorted([
        '2015-06-19 07:31:41.026 7947 INFO cinder.volume.manager [-] Updating volume status',
        '2015-11-27 11:23:07.146 16656 INFO cinder.volume.manager [req-a8c22cdb-e21b-497f-affe-9380478decae - - - - -] Updating volume status',
        '2015-11-27 11:23:07.148 16656 WARNING cinder.volume.manager [req-a8c22cdb-e21b-497f-affe-9380478decae - - - - -] Unable to update stats, LVMISCSIDriver -2.0.0 (config name rbd) driver is uninitialized.',
    ])

    assert sorted([i['raw_message'] for i in list(log.get_after(datetime(2015, 11, 27, 11, 23, 0)))]) == sorted([
        '2015-11-27 11:23:07.146 16656 INFO cinder.volume.manager [req-a8c22cdb-e21b-497f-affe-9380478decae - - - - -] Updating volume status',
        '2015-11-27 11:23:07.148 16656 WARNING cinder.volume.manager [req-a8c22cdb-e21b-497f-affe-9380478decae - - - - -] Unable to update stats, LVMISCSIDriver -2.0.0 (config name rbd) driver is uninitialized.',
    ])
